#!/bin/bash
################################################################################
################################################################################
#
# Program : AutoGenehunter.sh - Automated genome-wide scan with Genehunter
# Version : 1.0.3
# Date    : 18.10.2006
# Author  : Tero Hiekkalinna
# Email   : Tero Hiekkalinna@Helsinki.FI (tero hiekkalinna@ktl.fi)
#
# COPYRIGHT (C) Tero Hiekkalinna 2003-2006
#
################################################################################
################################################################################

#
# Variables
#
GENEHUNTER_COMMAND="gh"
XGENEHUNTER_COMMAND="xgh"
GENEHUNTER_COMMAND_FILE_SINGLE="gh_single.in"
GENEHUNTER_COMMAND_FILE_MULTI="gh_multi.in"
GENEHUNTER_SCREEN_FILE="gh_screen.out"
RESULTS_FOLDER="all_results"
PEDCONVERT_COMMAND="pedconvert"
PEDCONVERT_EXTRA_OPTIONS=" --prog genehunter --discrete "
DOWNCODE_COMMAND="downcode"
DOWNCODE_OPTIONS=" --prog genehunter "
MODEL_FILE="model.dat"
MALES=0
FEMALES=0
MAKE_FILES_ONLY=0
START_CHROMOSOME=1
STOP_CHROMOSOME=22
PEDIGREE_FILE_EXTENSION="raw"
PHENOTYPE_FILE="phen.dat"
GENEHUNTER_OVERRIDE=0
GENEHUNTER_OVERRIDE_FILE=""
GENEHUNTER_ANALYSIS=0
GENEHUNTER_BITS=16
GENEHUNTER_PS=0
SKIP_LARGE_OFF=0
PEDCHECK_COMMAND="pedcheck"
PEDCHECK_OPTIONS=""
PEDCHECK_LEVEL=2
PEDCHECK_NAMES_FILE="names.dat"
PEDCHECK_SKIP=0
MENDELIAN_ERRORS_FOUND=0
DEBUG=0
VERBOSE=0

#
# Program info & copyright notice
#
function info(){
   echo "################################################################################"
   echo "# Program : AutoGenehunter.sh - Automated genome-wide scan with Genehunter     #"
   echo "# Version : 1.0.3                                                              #"
   echo "# Date    : 18.10.2006                                                         #"
   echo "# Author  : Tero Hiekkalinna                                                   #"
   echo "# Email   : Tero Hiekkalinna@Helsinki.FI (tero hiekkalinna@ktl.fi)             #"
   echo "#                                                                              #"
   echo "# COPYRIGHT (C) Tero Hiekkalinna 2003-2006                                     #"
   echo "################################################################################"
}

#
# Usage
#
function usage(){
   info
   analysis_options
   echo
   echo "Usage: autogh.sh [OPTION]..."
   echo
   echo "Analysis options:"
   echo "================="
   echo
   echo " --chr [start] [stop]"
   echo "    Specify range of chromosomes to be analysed. Chromosome X is 23 and" 
   echo "    default range is 1-22"
   echo " --analysis [single,multi]"
   echo "    Specify single or multipoint analysis only, default is both"
   echo " --only [males,females]"
   echo "    Sex specific analysis, phenotypes of females are set to unknown and" 
   echo "    vice versa"
   echo " --bits [number]"
   echo "    Set bits for Genehunter analysis. Default value is $GENEHUNTER_BITS"  
   echo " --skip-large-off"
   echo "    Set \"skip large off\" option on Genehunter analysis"     
   echo " --gh-over [file]"
   echo "    This will override all Genehunter analysis options, all genehunter commands" 
   echo "    are read from file"
   echo " --gh-cmd [name]"
   echo "    Specify genehunter command (Default is $GENEHUNTER_COMMAND)"
   echo " --xgh-cmd [name]"
   echo "    Specify X-chromosome genehunter command (Default is $XGENEHUNTER_COMMAND)"
   echo " --pedcheck-cmd [name]"
   echo "    Specify pedcheck command (Default is $PEDCHECK_COMMAND)"
   echo " --skip-pedcheck"
   echo "    Skip Mendelian check (usefull if genotype data is known to be OK and"
   echo "    pedigrees are big!)"
   echo " --pedcheck-level [level]"
   echo "    Specify PedCheck check level (Default is $PEDCHECK_LEVEL)"
   echo
   echo "Input/Output file control options:"
   echo "=================================="
   echo
   echo " --phen [file]"
   echo "    Specify your own phenotype file name. Default is phen.dat"
   echo " --model [file]"
   echo "    Specify your own model file name. Default is model.dat"
   echo " --ignore"
   echo "    Pedigree file includes discrete trait as a first phenotype. It will be"
   echo "    read and ignored"
   echo " --liab"
   echo "    Phenotype file contains liability class after trait phenotype"
   echo " --ext [extension]"
   echo "    Pedigree file extension, example: raw or pre. Default is raw"
   echo " --makefilesonly"
   echo "    Make only Genehunter input files. Does not run analysis."
   echo " --ps"
   echo "    Toggle on Genehunter post script output option"
   echo
   echo "Help:"
   echo "====="
   echo
   echo " --help or -h"
   echo "    Print this help. See AUTOGSCAN for full documentation."
   echo
}

#
# debug
#
function debug(){
   echo "################################################################################"
   echo "PEDCONVERT_EXTRA_OPTIONS=$PEDCONVERT_EXTRA_OPTIONS"
   echo "MALES=$MALES"
   echo "FEMALES=$FEMALES"
   echo "MAKE_FILES_ONLY=$MAKE_FILES_ONLY"
   echo "PEDIGREE_FILE_EXTENSION=$PEDIGREE_FILE_EXTENSION"
   echo "START_CHROMOSOME=$START_CHROMOSOME"
   echo "STOP_CHROMOSOME=$STOP_CHROMOSOME"
   echo "PHENOTYPE_FILE=$PHENOTYPE_FILE"
   echo "GENEHUNTER_OVERRIDE=$GENEHUNTER_OVERRIDE"
   echo "GENEHUNTER_OVERRIDE_FILE=$GENEHUNTER_OVERRIDE_FILE"
   echo "GENEHUNTER_BITS=$GENEHUNTER_BITS"
   echo "GENEHUNTER_ANALYSIS=$GENEHUNTER_ANALYSIS 0=single and multi,1=single,2=multi"
   echo "DEBUG=$DEBUG"
   echo "VERBOSE=$VERBOSE"
   echo "################################################################################"
   exit
}

#
# Prints used options
#
function analysis_options(){
   echo "################################################################################"
   echo "# Default options are: "
   echo "# Results folder: `pwd`/$RESULTS_FOLDER "
   echo "################################################################################"
}

#
# Convert input files to Unix format
#
function convert_to_unix_format(){
   dos2unix -q -V >& /dev/null
   echo "--------------------------------------------------------------------------------"
   if [ $? != '0' ]
   then
      echo " Warning: Program dos2unix not found!"
   else
      echo " Converting input files to Unix format!"
      for INPUTFILE in $PEDIGREEFILE $PHENOTYPE_FILE $MAPFILE
      do
         dos2unix -q $INPUTFILE
      done
   fi
   echo "--------------------------------------------------------------------------------"
}

#
# Mendelian check
#
function mendelian_check(){
   echo "--------------------------------------------------------------------------------"
   echo " PedCheck "
   echo "--------------------------------------------------------------------------------"
   if [ -f pedcheck.err ]
   then
      mv pedcheck.err pedcheck.err.old
   fi
   MENDELIAN_ERRORS_FOUND=0
   if [ $CHROMOSOME -eq 23 ]
   then
      PEDCHECK_OPTIONS=$PEDCHECK_OPTIONS" -x "
   fi
   $PEDCHECK_COMMAND -n $PEDCHECK_NAMES_FILE -p ../$PEDIGREEFILE -$PEDCHECK_LEVEL $PEDCHECK_OPTIONS >& /dev/null
   if [ -f pedcheck.err ]
   then
      GENOTYPE_ERROR_COUNT=`grep -c "GENOTYPE ERROR" pedcheck.err`
      if [ $GENOTYPE_ERROR_COUNT -ne 0 ]
      then
         MENDELIAN_ERRORS_FOUND=1
      else
         echo "Mendelian check passed (level $PEDCHECK_LEVEL)..."
      fi
   else
      echo "Warning: pedcheck.err file not found, is pedcheck installed?"
   fi
}

#
# MAIN
#

#
# Parse command line arguments
#
while [ $# -ge 1 ]; do
   case "$1" in
      --only)
         if [ $2 == "males" ]
         then
            MALES=1
            PEDCONVERT_EXTRA_OPTIONS=$PEDCONVERT_EXTRA_OPTIONS" --only males "
         elif [ $2 == "females" ]
         then
            FEMALES=1
            PEDCONVERT_EXTRA_OPTIONS=$PEDCONVERT_EXTRA_OPTIONS" --only females "
         else
            usage
            echo
            echo "Error: argument after --only must be males or females!"
            echo
            exit 0
         fi
         shift
         ;;
      --makefilesonly)
         MAKE_FILES_ONLY=1
         ;;
      --ignore)
         DICHOTOMOUS_TRAIT=1
         PEDCONVERT_EXTRA_OPTIONS=$PEDCONVERT_EXTRA_OPTIONS" --ignore "
         ;;
      --ext)
         PEDIGREE_FILE_EXTENSION=$2
         shift
         ;;
      --gh-over)
         if [ -f $2 ]
         then
            GENEHUNTER_OVERRIDE=1
            GENEHUNTER_OVERRIDE_FILE=$2
         else
            info
            echo
            echo "Error: Genehunter override file $2 not found!"
            echo
            exit 1
         fi
         shift
         ;;
      --chr)
         START_CHROMOSOME=$2
         STOP_CHROMOSOME=$3
         if [ $START_CHROMOSOME -lt 1 ] || [ $START_CHROMOSOME -gt 23 ]
         then
            usage
            echo
            echo "Error: Illegal start chromosome number: $START_CHROMOSOME"
            echo
            exit 1
         elif [ $STOP_CHROMOSOME -lt 1 ] || [ $STOP_CHROMOSOME -gt 23 ]
         then
            usage
            echo
            echo "Error: Illegal stop chromosome number: $STOP_CHROMOSOME"
            echo
            exit 1
         elif [ $START_CHROMOSOME -gt $STOP_CHROMOSOME ]
         then
            usage
            echo
            echo "Error: Start chromosome $START_CHROMOSOME is greater than stop chromosome $STOP_CHROMOSOME!"
            echo
            exit 1
         fi
         shift; shift
         ;;
      --phen)
         if [ -f $2 ]
         then
            PHENOTYPE_FILE=$2
         else
            info
            echo
            echo "Error: Phenotype file $2 not found!"
            echo
            exit 1
         fi
         shift
         ;;
       --model)
         if [ -f $2 ]
         then
            MODEL_FILE=$2
         else
            info
            echo
            echo "Error: Model file $2 not found!"
            echo
            exit 1
         fi
         shift
         ;;
      --analysis)
         if [ $2 == "single" ]
         then
            GENEHUNTER_ANALYSIS=1
         elif [ $2 == "multi" ]
         then
            GENEHUNTER_ANALYSIS=2
         else
            usage
            echo
            echo "Error: argument after --analysis must be single or multi!"
            echo
            exit 1
         fi
         shift
         ;;
      --bits)
         GENEHUNTER_BITS=$2
         shift
         ;;
      --skip-large-off)
         SKIP_LARGE_OFF=1
         ;;
      --liab)
         DOWNCODE_OPTIONS=$DOWNCODE_OPTIONS" --liab "
         ;;
      --gh-cmd)
         GENEHUNTER_COMMAND=$2
         shift
         ;;
      --xgh-cmd)
         XGENEHUNTER_COMMAND=$2
         shift
         ;;
      --pedcheck-cmd)
         PEDCHECK_COMMAND=$2
         shift
         ;;
      --skip-pedcheck)
         PEDCHECK_SKIP=1
         ;;
      --pedcheck-level)
         PEDCHECK_LEVEL=$2
         shift
         ;;
      --ps)
         GENEHUNTER_PS=1
         ;;
      --debug)
         DEBUG=1
         ;;
      --verbose)
         VERBOSE=1
         ;;
      --help|-h)
         usage
         exit 1
         ;;
      *)
         usage
         echo "Error: Unknown option:" $1
         exit 1
         ;;
   esac
   shift
done

info
analysis_options

#
# verbose debug
#
if [ $DEBUG -eq 1 ]
then
   debug
fi

#
# Create results folder
#
if [ ! -d $RESULTS_FOLDER ]
then
   mkdir $RESULTS_FOLDER
fi

#
# Chromosomes...
#
for ((CHROMOSOME=$START_CHROMOSOME ; CHROMOSOME <=$STOP_CHROMOSOME  ; CHROMOSOME++))
do

   echo "################################################################################"
   echo "# CHROMOSOME $CHROMOSOME"
   echo "################################################################################"

   #
   # Input files
   #
   PEDIGREEFILE="chr"$CHROMOSOME"."$PEDIGREE_FILE_EXTENSION
   MAPFILE="map"$CHROMOSOME".dat"

   #
   # CHROMOSOME-X
   #
   if [ $CHROMOSOME -eq 23 ]
   then
      GENEHUNTER_COMMAND=$XGENEHUNTER_COMMAND
      MODEL_FILE="xmodel.dat"
      DOWNCODE_OPTIONS=$DOWNCODE_OPTIONS" --x "
   fi

   #
   # If pedigree file exists, start analysis
   #
   if [ -f $PEDIGREEFILE ] && [ -f $MAPFILE ] && [ -f $MODEL_FILE ] && [ -f $PHENOTYPE_FILE ]
   then

      #
      # Count number of markers from pedigree file
      #
      if [ $DICHOTOMOUS_TRAIT -eq 1 ]
      then
         NUMBER_OF_LOCI=`awk 'END {print (NF - 6)/2}' $PEDIGREEFILE`
      else
         NUMBER_OF_LOCI=`awk 'END {print (NF - 5)/2}' $PEDIGREEFILE`
      fi

      #
      # Chromosome directory
      #
	   CHROMOSOMEDIR="chr"$CHROMOSOME
      if [ ! -d $CHROMOSOMEDIR ]
      then
         mkdir $CHROMOSOMEDIR
      fi

	   #
      # Output files
      #
      if [ $MALES -eq 1 ]
      then
         GENEHUNTER_SINGLEPOINT_OUT="gh_singlepoint_"$CHROMOSOME"_males.out"
         GENEHUNTER_MULTIPOINT_OUT="gh_multipoint_"$CHROMOSOME"_males.out"
         GENEHUNTER_MULTIPOINT_LOD_PS="gh_multipoint_"$CHROMOSOME"_males_lod.ps"
         GENEHUNTER_MULTIPOINT_NPL_PS="gh_multipoint_"$CHROMOSOME"_males_npl.ps"
         GENEHUNTER_MULTIPOINT_INFO_PS="gh_multipoint_"$CHROMOSOME"_males_info.ps"
      elif [ $FEMALES -eq 1 ]
	   then
         GENEHUNTER_SINGLEPOINT_OUT="gh_singlepoint_"$CHROMOSOME"_females.out"
         GENEHUNTER_MULTIPOINT_OUT="gh_multipoint_"$CHROMOSOME"_females.out"
         GENEHUNTER_MULTIPOINT_LOD_PS="gh_multipoint_"$CHROMOSOME"_females_lod.ps"
         GENEHUNTER_MULTIPOINT_NPL_PS="gh_multipoint_"$CHROMOSOME"_females_npl.ps"
         GENEHUNTER_MULTIPOINT_INFO_PS="gh_multipoint_"$CHROMOSOME"_females_info.ps"
      else
         GENEHUNTER_SINGLEPOINT_OUT="gh_singlepoint_"$CHROMOSOME".out"
         GENEHUNTER_MULTIPOINT_OUT="gh_multipoint_"$CHROMOSOME".out"
         GENEHUNTER_MULTIPOINT_LOD_PS="gh_multipoint_"$CHROMOSOME"_lod.ps"
         GENEHUNTER_MULTIPOINT_NPL_PS="gh_multipoint_"$CHROMOSOME"_npl.ps"
         GENEHUNTER_MULTIPOINT_INFO_PS="gh_multipoint_"$CHROMOSOME"_info.ps"
      fi

      echo "--------------------------------------------------------------------------------"
      echo " NUMBER OF LOCI $NUMBER_OF_LOCI "
      echo "--------------------------------------------------------------------------------"

	   if [ -d $CHROMOSOMEDIR ]
	   then
         #
         # Change to chromosome directory
         #
	      cd $CHROMOSOMEDIR

         #
         # PedConvert
         #
         $PEDCONVERT_COMMAND -p ../$PEDIGREEFILE -f ../$PHENOTYPE_FILE  -m ../$MAPFILE -n $NUMBER_OF_LOCI $PEDCONVERT_EXTRA_OPTIONS 

         #
         # Mendelian check
         # 
         if [ $PEDCHECK_SKIP -eq 0 ]
         then
            mendelian_check
         fi
         if [ $MENDELIAN_ERRORS_FOUND -eq 0 ]
         then        
            #
            # Genehunter input files
            #
            GENEHUNTER_PEDIGREEFILE="gh.ped"
            GENEHUNTER_LOCUSFILE="gh.dat"

            #
            # DownCode
            #
            $DOWNCODE_COMMAND $DOWNCODE_OPTIONS -p $GENEHUNTER_PEDIGREEFILE -map ../$MAPFILE -m ../$MODEL_FILE -n $NUMBER_OF_LOCI --trait --pre 
            GENEHUNTER_PEDIGREEFILE="gh_coded.ped"         
            #GENEHUNTER_PEDIGREEFILE_CODED="gh_coded_"$CHROMOSOME".ped"
            #mv -fv gh_coded.ped $GENEHUNTER_PEDIGREEFILE_CODED
            #GENEHUNTER_PEDIGREEFILE=$GENEHUNTER_PEDIGREEFILE_CODED

            #
            # Genehunter command files
            #
            # Singlepoint
            #
            echo "photo "$GENEHUNTER_SCREEN_FILE > $GENEHUNTER_COMMAND_FILE_SINGLE
            echo "load markers "$GENEHUNTER_LOCUSFILE >> $GENEHUNTER_COMMAND_FILE_SINGLE
            echo "max bits "$GENEHUNTER_BITS >> $GENEHUNTER_COMMAND_FILE_SINGLE
            echo "single on" >> $GENEHUNTER_COMMAND_FILE_SINGLE
            if [ $SKIP_LARGE_OFF -eq 1 ]
            then
               echo "skip large off" >> $GENEHUNTER_COMMAND_FILE_SINGLE
            fi            
            echo "scan pedigrees "$GENEHUNTER_PEDIGREEFILE >> $GENEHUNTER_COMMAND_FILE_SINGLE
            echo "photo off" >> $GENEHUNTER_COMMAND_FILE_SINGLE
            echo "photo "$GENEHUNTER_SINGLEPOINT_OUT >> $GENEHUNTER_COMMAND_FILE_SINGLE
            echo "total stat het" >> $GENEHUNTER_COMMAND_FILE_SINGLE
            #
            # Multipoint
            #
            echo "photo "$GENEHUNTER_SCREEN_FILE > $GENEHUNTER_COMMAND_FILE_MULTI
            echo "load markers "$GENEHUNTER_LOCUSFILE >> $GENEHUNTER_COMMAND_FILE_MULTI
            echo "max bits "$GENEHUNTER_BITS >> $GENEHUNTER_COMMAND_FILE_MULTI
            if [ $SKIP_LARGE_OFF -eq 1 ]
            then
               echo "skip large off" >> $GENEHUNTER_COMMAND_FILE_MULTI
            fi                       
            echo "scan pedigrees "$GENEHUNTER_PEDIGREEFILE >> $GENEHUNTER_COMMAND_FILE_MULTI
            echo "photo off" >> $GENEHUNTER_COMMAND_FILE_MULTI
            echo "photo "$GENEHUNTER_MULTIPOINT_OUT >> $GENEHUNTER_COMMAND_FILE_MULTI
            if [ $GENEHUNTER_PS -eq 1 ]
            then
              echo "post on" >> $GENEHUNTER_COMMAND_FILE_MULTI
            fi
            echo "total stat het" >> $GENEHUNTER_COMMAND_FILE_MULTI
            if [ $GENEHUNTER_PS -eq 1 ]
            then
               echo $GENEHUNTER_MULTIPOINT_NPL_PS >> $GENEHUNTER_COMMAND_FILE_MULTI
               echo $GENEHUNTER_MULTIPOINT_LOD_PS >> $GENEHUNTER_COMMAND_FILE_MULTI
               echo $GENEHUNTER_MULTIPOINT_INFO_PS >> $GENEHUNTER_COMMAND_FILE_MULTI
            fi


            #
            # Genehunter analysis
            #
            if [ $MAKE_FILES_ONLY -eq 0 ]
            then
               echo "################################################################################"
               echo "# Genehunter (ANALYSIS)"
               echo "################################################################################"
               if [ -f $GENEHUNTER_PEDIGREEFILE ] && [ -f $GENEHUNTER_LOCUSFILE ] && [ -f $GENEHUNTER_COMMAND_FILE_SINGLE ] && [ -f $GENEHUNTER_COMMAND_FILE_MULTI ]
               then
                  if [ $GENEHUNTER_ANALYSIS -eq 0 ] || [ $GENEHUNTER_ANALYSIS -eq 1 ]
                  then
                     echo "Singlepoint analysis...commands used:"
                     echo "====================================="
                     cat $GENEHUNTER_COMMAND_FILE_SINGLE
                     echo "Analyzing..."
                     $GENEHUNTER_COMMAND < $GENEHUNTER_COMMAND_FILE_SINGLE >& /dev/null
                     echo "...done"
                  fi
                  #
                  echo "################################################################################"
                  if [ $GENEHUNTER_ANALYSIS -eq 0 ] || [ $GENEHUNTER_ANALYSIS -eq 2 ]
                  then
                     echo "Multipoint analysis...commands used:"
                     echo "===================================="
                     cat $GENEHUNTER_COMMAND_FILE_MULTI
                     echo "Analyzing..."
                     $GENEHUNTER_COMMAND < $GENEHUNTER_COMMAND_FILE_MULTI >& /dev/null
                     echo "...done"
                  fi
               else
                  echo "Warning: Input files are missing: $GENEHUNTER_PEDIGREEFILE, $GENEHUNTER_LOCUSFILE, $GENEHUNTER_COMMAND_FILE_SINGLE, $GENEHUNTER_COMMAND_FILE_MULTI"
               fi
               #
               # Move results
               #
               echo
               echo "Moving results files if any..."
               echo
               for OUTPUTFILE in $GENEHUNTER_MULTIPOINT_OUT $GENEHUNTER_SINGLEPOINT_OUT $GENEHUNTER_MULTIPOINT_LOD_PS $GENEHUNTER_MULTIPOINT_NPL_PS $GENEHUNTER_MULTIPOINT_INFO_PS
               do
                  if [ -f $OUTPUTFILE ]
                  then
                     mv -v $OUTPUTFILE ../$RESULTS_FOLDER
                  fi
               done
               #
               # Compress large files
               #
               echo
               echo "Compressing $GENEHUNTER_SCREEN_FILE file to safe disc space..."
               gzip -vf $GENEHUNTER_SCREEN_FILE        
            fi
         else
            echo "--------------------------------------------------------------------------------"
            echo " Mendelian error(s) found, see pedcheck.err, skipping this chromosome..."
            echo "--------------------------------------------------------------------------------"         
         fi
         cd ..     
      fi
   else
      echo " Skipping...no pedigree file $PEDIGREEFILE or map file $MAPFILE or model file $MODEL_FILE or phenotype file $PHENOTYPE_FILE found! "
   fi
done

echo
echo "End of the AutoGenehunter!"
echo
