#!/bin/bash
################################################################################
################################################################################
#
# Program : AutoMerlin.sh - Automated genome-wide scan with Merlin
# Version : 1.0.3
# Date    : 18.10.2006
# Author  : Tero Hiekkalinna
# Email   : Tero Hiekkalinna@Helsinki.FI (tero hiekkalinna@ktl.fi)
#
# COPYRIGHT (C) Tero Hiekkalinna 2003-2006
#
################################################################################
################################################################################

#
# Variables
#
MERLIN_COMMAND="merlin"
XMERLIN_COMMAND="minx"
MERLIN_OPTIONS=" --quiet --markerNames "
RESULTS_FOLDER="all_results"
PEDCONVERT_COMMAND="pedconvert"
PEDCONVERT_EXTRA_OPTIONS=" --prog merlin "
COVARIATES=0
SEX=0
MALES=0
FEMALES=0
MAKE_FILES_ONLY=0
DICHOTOMOUS_TRAIT=0
PEDIGREE_FILE_EXTENSION="raw"
MERLIN_EXTRA_OPTIONS=0
MERLIN_EXTRA_OPTIONS_FILE=""
START_CHROMOSOME=1
STOP_CHROMOSOME=22
PHENOTYPE_FILE="phen.dat"
MODEL_FILE="model.dat"
MERLIN_OVERRIDE=0
MERLIN_OVERRIDE_FILE=""
MERLIN_ANALYSIS=0
MERLIN_WIPED=0
MERLIN_ERROR=0
MERLIN_BITS=40
MERLIN_MISSING=" x "
MERLIN_SIMULATE=0
MERLIN_REPLICATES=1
MERLIN_NPL=0
MERLIN_MODEL=0
MERLIN_PDF=0
MERLIN_TRIM=0
MERLIN_VC=0
PEDWIPE_COMMAND="pedwipe"
PEDSTATS_COMMAND="pedstats"
PEDSTATS=0
PEDSTATS_OPTIONS=" --hardyWeinberg --checkAll --showAll "
DISABLE_MAP_SET=0
DISABLE_MAP_POSITION="foo"
PEDCHECK_COMMAND="pedcheck"
PEDCHECK_OPTIONS=""
PEDCHECK_LEVEL=2
PEDCHECK_NAMES_FILE="names.dat"
PEDCHECK_SKIP=0
MENDELIAN_ERRORS_FOUND=0
DEBUG=0
VERBOSE=0

#
# Program info & copyright notice
#
function info(){
   echo "################################################################################"
   echo "# Program : AutoMerlin.sh - Automated genome-wide scan with Merlin             #"
   echo "# Version : 1.0.3                                                              #"
   echo "# Date    : 18.10.2006                                                         #"
   echo "# Author  : Tero Hiekkalinna                                                   #"
   echo "# Email   : Tero Hiekkalinna@Helsinki.FI (tero hiekkalinna@ktl.fi)             #"
   echo "#                                                                              #"
   echo "# COPYRIGHT (C) Tero Hiekkalinna 2003-2006                                     #"
   echo "################################################################################"
}

#
# Usage
#
function usage(){
   info
   analysis_options
   echo
   echo "Usage: automerlin.sh [OPTION]..."
   echo
   echo "Quantitative trait specific options:"
   echo "===================================="  
   echo
   echo " --vc"
   echo "    Variance Component linkage analysis"
   echo " --cov"
   echo "    Use covariates, first phenotype in phenotype file is selected as trait and"
   echo "    rest of the phenotypes are covariates, otherwise all phenotypes are"
   echo "    analyzed separately"
   echo " --sex"
   echo "    Add sex as covariate"
   echo
   echo "Qualitative (discrete) trait specific options:"
   echo "=============================================="
   echo
   echo " --npl (same as --discrete)"              
   echo "    Non-parametric ('model-free') linkage analysis"
   echo " --model [file]"              
   echo "    Parametric ('model-based') linkage analysis"
   echo
   echo "Other analysis options:"
   echo "======================="
   echo
   echo " --chr [start] [stop]"
   echo "    Specify range of chromosomes to be analysed. Chromosome X is 23 and" 
   echo "    default range is 1-22"
   echo " --only [males,females]"
   echo "    Sex specific analysis, phenotypes of females are set to unknown and" 
   echo "    vice versa"
   echo " --analysis [single,multi]"
   echo "    Specify single or multipoint analysis only, default is both"
   echo " --trim"
   echo "    Toggle on MERLIN --trim option"
   echo " --information"
   echo "    Toggle on MERLIN --information option"
   echo " --pedstats"
   echo "    Pedigree statistics analysis (PedStats)"
   echo " --error"
   echo "    Genotyping error analysis"
   echo " --wiped"
   echo "    Performs scan based on --error analysis. Error analysis must be done first"
   echo " --bits [number]"
   echo "    Set bits for Merlin analysis. Default value is $MERLIN_BITS"
   echo " --missing [variable]"
   echo "    Specify missing value. Default value is $MERLIN_MISSING"
   echo " --simulate"
   echo "    Genome-wide scan simulation"
   echo " --replicates [number]"
   echo "    Specify number of replicates for simulation. Default value is $MERLIN_REPLICATES"
   echo " --merlin-extra [file]"
   echo "    Extra analysis option file for Merlin. All extra commands have to be in"
   echo "    first line of the file"
   echo " --merlin-over [file]"
   echo "    This will override all Merlin analysis options, all merlin commands have" 
   echo "    to be first line of the file"
   echo " --disable-map-position [number (cM)]" 
   echo "    Disable specific marker from analysis with some map position. This will" 
   echo "    write S2 to MERLIN locus file on that marker."
   echo " --merlin-cmd [name]"
   echo "    Specify merlin command (Default is $MERLIN_COMMAND)"
   echo " --xmerlin-cmd [name]"
   echo "    Specify X-chromosome merlin command (Default is $XMERLIN_COMMAND)"
   echo " --pedcheck-cmd [name]"
   echo "    Specify pedcheck command (Default is $PEDCHECK_COMMAND)"
   echo " --skip-pedcheck"
   echo "    Skip Mendelian check (usefull if genotype data is known to be OK and"
   echo "    pedigrees are big!)"
   echo " --pedcheck-level [level]"
   echo "    Specify PedCheck check level (Default is $PEDCHECK_LEVEL)"
   echo
   echo "Input/Output file control options:"
   echo "=================================="
   echo
   echo " --phen [file]"
   echo "    Specify your own phenotype file name. Default is phen.dat"
   echo " --ignore"
   echo "    Pedigree file includes discrete trait as a first phenotype. It will be"
   echo "    read and ignored"
   echo " --ext [extension]"
   echo "    Pedigree file extension, example: raw or pre. Default is raw"
   echo " --makefilesonly"
   echo "    Make only Merlin input files. Does not run analysis."
   echo " --pdf"
   echo "    Toggle on MERLIN --pdf option"
   echo
   echo "Help:"
   echo "====="
   echo
   echo " --help or -h"
   echo "    Print this help. See AUTOGSCAN for full documentation."
   echo
}

#
# debug
#
function debug(){
   echo "################################################################################"
   echo "PEDCONVERT_EXTRA_OPTIONS=$PEDCONVERT_EXTRA_OPTIONS"
   echo "COVARIATES=$COVARIATES"
   echo "SEX=$SEX"
   echo "MALES=$MALES"
   echo "FEMALES=$FEMALES"
   echo "MAKE_FILES_ONLY=$MAKE_FILES_ONLY"
   echo "DICHOTOMOUS_TRAIT=$DICHOTOMOUS_TRAIT"
   echo "PEDIGREE_FILE_EXTENSION=$PEDIGREE_FILE_EXTENSION"
   echo "MERLIN_COMMAND=$MERLIN_COMMAND"
   echo "MERLIN_EXTRA_OPTIONS=$MERLIN_EXTRA_OPTIONS"
   echo "MERLIN_EXTRA_OPTIONS_FILE=$MERLIN_EXTRA_OPTIONS_FILE"
   echo "START_CHROMOSOME=$START_CHROMOSOME"
   echo "STOP_CHROMOSOME=$STOP_CHROMOSOME"
   echo "PHENOTYPE_FILE=$PHENOTYPE_FILE"
   echo "MERLIN_OVERRIDE=$MERLIN_OVERRIDE"
   echo "MERLIN_OVERRIDE_FILE=$MERLIN_OVERRIDE_FILE"
   echo "MERLIN_WIPED=$MERLIN_WIPED"
   echo "MERLIN_ERROR=$MERLIN_ERROR"
   echo "MERLIN_BITS=$MERLIN_BITS"
   echo "MERLIN_MISSING=$MERLIN_MISSING"
   echo "MERLIN_SIMULATE=$MERLIN_SIMULATE"
   echo "MERLIN_REPLICATES=$MERLIN_REPLICATES"
   echo "PEDWIPE_COMMAND=$PEDWIPE_COMMAND"
   echo "PEDSTATS_COMMAND=$PEDSTATS_COMMAND"
   echo "MERLIN_ANALYSIS=$MERLIN_ANALYSIS (0=single and multi),(1=single),(2=multi)"
   echo "PEDSTATS=$PEDSTATS"
   echo "PEDSTATS_OPTIONS=$PEDSTATS_OPTIONS"
   echo "DEBUG=$DEBUG"
   echo "VERBOSE=$VERBOSE"
   echo "################################################################################"
   exit
}

#
# Prints used merlin options
#
function analysis_options(){
   echo "################################################################################"
   echo "# Default merlin options are: "
   echo "# -x $MERLIN_MISSING $MERLIN_OPTIONS --bits $MERLIN_BITS"
   echo "# Default pedstats options are: "
   echo "# $PEDSTATS_OPTIONS "
   echo "# Results folder: `pwd`/$RESULTS_FOLDER "
   echo "################################################################################"
}

#
# Check merlin version
#
function check_merlin(){
   MAJOR=`merlin | head -1 | cut -d" " -f2 | cut -d"." -f2`
   MINOR=`merlin | head -1 | cut -d" " -f2 | cut -d"." -f1`
   if [ $MAJOR -lt 10 ] && [ $MINOR -lt 1 ]
   then
      echo
      echo "Error: Merlin version must be atleast 0.10.1"
      echo
      exit 1
   fi
}

#
# Check input files
#
function check_files(){
   if [ ! -f $PHENOTYPE_FILE ]
   then
      info
      echo
      echo "Error: Phenotype file $PHENOTYPE_FILE is missing from directory: `pwd`"
      echo
      exit 1
   fi
   if [ ! -f $MODEL_FILE ]
   then
      info
      echo
      echo "Error: Parametric model file $MODEL_FILE is missing from directory: `pwd`"
      echo
      exit 1
   fi
}

#
# Mendelian check
#
function mendelian_check(){
   echo "--------------------------------------------------------------------------------"
   echo " PedCheck "
   echo "--------------------------------------------------------------------------------"
   if [ -f pedcheck.err ]
   then
      mv pedcheck.err pedcheck.err.old
   fi
   MENDELIAN_ERRORS_FOUND=0
   if [ $CHROMOSOME -eq 23 ]
   then
      PEDCHECK_OPTIONS=$PEDCHECK_OPTIONS" -x "
   fi
   
   # Chr greater than 23
   if [ $CHROMOSOME -gt 23 ]
   then
      PEDCHECK_OPTIONS=" "
   fi

   $PEDCHECK_COMMAND -n $PEDCHECK_NAMES_FILE -p ../$PEDIGREEFILE -$PEDCHECK_LEVEL $PEDCHECK_OPTIONS >& /dev/null
   if [ -f pedcheck.err ]
   then
      GENOTYPE_ERROR_COUNT=`grep -c "GENOTYPE ERROR" pedcheck.err`
      if [ $GENOTYPE_ERROR_COUNT -ne 0 ]
      then
         MENDELIAN_ERRORS_FOUND=1
      else
         echo "Mendelian check passed (level $PEDCHECK_LEVEL)..."
      fi
   else
      echo "Warning: pedcheck.err file not found, is pedcheck installed?"
   fi
}

#
# Convert input files to Unix format
#
function convert_to_unix_format(){
   dos2unix -q -V >& /dev/null
   echo "--------------------------------------------------------------------------------"
   if [ $? != '0' ]
   then
      echo " Warning: Program dos2unix not found!"
   else
      echo " Converting input files to Unix format!"
      for INPUTFILE in $PEDIGREEFILE $PHENOTYPE_FILE $MAPFILE
      do
         dos2unix -q $INPUTFILE
      done
   fi
   echo "--------------------------------------------------------------------------------"
}

#
# MAIN
#

#
# Parse command line arguments
#
while [ $# -ge 1 ]; do
   case "$1" in
      --cov)
         COVARIATES=1
         MERLIN_OPTIONS=$MERLIN_OPTIONS" --useCovariates "
         ;;
      --sex)
         SEX=1
         PEDCONVERT_EXTRA_OPTIONS=$PEDCONVERT_EXTRA_OPTIONS" --sex "
         ;;
      --only)
         if [ $2 == "males" ]
         then
            MALES=1
            PEDCONVERT_EXTRA_OPTIONS=$PEDCONVERT_EXTRA_OPTIONS" --only males "
         elif [ $2 == "females" ]
         then
            FEMALES=1
            PEDCONVERT_EXTRA_OPTIONS=$PEDCONVERT_EXTRA_OPTIONS" --only females "
         else
            usage
            echo
            echo "Error: argument after --only must be males or females!"
            echo
            exit 0
         fi
         shift
         ;;
      --makefilesonly)
         MAKE_FILES_ONLY=1
         ;;
      --ignore)
         DICHOTOMOUS_TRAIT=1
         PEDCONVERT_EXTRA_OPTIONS=$PEDCONVERT_EXTRA_OPTIONS" --ignore "
         ;;
      --ext)
         PEDIGREE_FILE_EXTENSION=$2
         shift
         ;;
      --merlin-extra)
         if [ -f $2 ]
         then
            MERLIN_EXTRA_OPTIONS=1
            MERLIN_EXTRA_OPTIONS_FILE=$2
            MERLIN_OPTIONS=$MERLIN_OPTIONS" `head -n 1 $MERLIN_EXTRA_OPTIONS_FILE` "
         else
            info
            echo
            echo "Error: Merlin extra options file $2 not found!"
            echo
            exit 1
         fi
         shift
         ;;
      --merlin-over)
         if [ -f $2 ]
         then
            MERLIN_OVERRIDE=1
            MERLIN_OVERRIDE_FILE=$2
            MERLIN_OPTIONS=`head -n 1 $MERLIN_OVERRIDE_FILE`
         else
            info
            echo
            echo "Error: Merlin override file $2 not found!"
            echo
            exit 1
         fi
         shift
         ;;
      --chr)
         START_CHROMOSOME=$2
         STOP_CHROMOSOME=$3
         if [ $START_CHROMOSOME -lt 1 ] || [ $START_CHROMOSOME -gt 24 ]
         then
            usage
            echo
            echo "Error: Illegal start chromosome number: $START_CHROMOSOME"
            echo
            exit 1
         elif [ $STOP_CHROMOSOME -lt 1 ] || [ $STOP_CHROMOSOME -gt 24 ]
         then
            usage
            echo
            echo "Error: Illegal stop chromosome number: $STOP_CHROMOSOME"
            echo
            exit 1
         elif [ $START_CHROMOSOME -gt $STOP_CHROMOSOME ]
         then
            usage
            echo
            echo "Error: Start chromosome $START_CHROMOSOME is greater than stop chromosome $STOP_CHROMOSOME!"
            echo
            exit 1
         fi
         shift; shift
         ;;
      --phen)
         if [ -f $2 ]
         then
            PHENOTYPE_FILE=$2
         else
            info
            echo
            echo "Error: Phenotype file $2 not found!"
            echo
            exit 1
         fi
         shift
         ;;
       --model)
         if [ -f $2 ]
         then
            if [ -z $2 ]
            then
               info
               echo "Error: Parametric model file name missing!"
               exit 1
            fi
            MERLIN_OPTIONS=$MERLIN_OPTIONS" --model param.tbl "
            MODEL_FILE=$2
            MERLIN_MODEL=1
            MERLIN_NPL=1
         else
            info
            echo
            echo "Error: Model file $2 not found!"
            echo
            exit 1
         fi
         shift
         ;;
      --analysis)
         if [ $2 == "single" ]
         then
            MERLIN_ANALYSIS=1
         elif [ $2 == "multi" ]
         then
            MERLIN_ANALYSIS=2
         else
            usage
            echo
            echo "Error: argument after --analysis must be single or multi!"
            echo
            exit 1
         fi
         shift
         ;;
      --pedstats)
         PEDSTATS=1
         ;;
      --wiped)
         MERLIN_WIPED=1
         ;;
      --error)
         MERLIN_ERROR=1
         ;;
      --bits)
         MERLIN_BITS=$2
         shift
         ;;
      --missing)
         MERLIN_MISSING=$2
         shift
         ;;
      --merlin-cmd)
         MERLIN_COMMAND=$2
         shift
         ;;
      --pedstats-cmd)
         PEDSTATS_COMMAND=$2
         shift
         ;;
      --pedwipe-cmd)
         PEDWIPE_COMMAND=$2
         shift
         ;;
      --simulate)
         MERLIN_SIMULATE=1
         ;;
      --replicates)
         MERLIN_REPLICATES=$2
         if [ $MERLIN_REPLICATES -lt 1 ] || [ $MERLIN_REPLICATES -gt 1000 ]
         then
            echo
            echo "Error: Number of replicates must be between 1 and 1000"
            echo
            exit 1
         fi
         shift
         ;;
      --disable-map-position)
         DISABLE_MAP_SET=1
         DISABLE_MAP_POSITION=$2
         shift
         ;;
      --discrete)
         MERLIN_OPTIONS=$MERLIN_OPTIONS" --npl "
         MERLIN_NPL=1
         ;;
      --npl)
         MERLIN_OPTIONS=$MERLIN_OPTIONS" --npl "
         MERLIN_NPL=1
         ;;
      --pdf)
         MERLIN_OPTIONS=$MERLIN_OPTIONS" --pdf "
         MERLIN_PDF=1
         ;;
      --trim)
         MERLIN_OPTIONS=$MERLIN_OPTIONS" --trim "
         MERLIN_TRIM=1
         ;;
      --vc)
         MERLIN_OPTIONS=$MERLIN_OPTIONS" --vc "
         MERLIN_VC=1
         ;;
      --information)
         MERLIN_OPTIONS=$MERLIN_OPTIONS" --information "
         ;;
      --pedcheck-cmd)
         PEDCHECK_COMMAND=$2
         shift
         ;;
      --skip-pedcheck)
         PEDCHECK_SKIP=1
         ;;
      --pedcheck-level)
         PEDCHECK_LEVEL=$2
         shift
         ;;     
      --debug)
         DEBUG=1
         ;;
      --verbose)
         VERBOSE=1
         ;;
      --help|-h)
         usage
         exit 1
         ;;
      *)
         usage
         echo "Error: Unknown option:" $1
         exit 1
         ;;
   esac
   shift
done

info
analysis_options

#
# verbose debug
#
if [ $DEBUG -eq 1 ]
then
   debug
fi

check_merlin
check_files

#
# Create results folder
#
if [ ! -d $RESULTS_FOLDER ]
then
   mkdir $RESULTS_FOLDER
fi

#
# Merlin options
#
MERLIN_OPTIONS=$MERLIN_OPTIONS" --bits "$MERLIN_BITS" -x "$MERLIN_MISSING

#
# Chromosomes...
#
for ((CHROMOSOME=$START_CHROMOSOME ; CHROMOSOME <=$STOP_CHROMOSOME  ; CHROMOSOME++))
do

   echo "################################################################################"
   echo "# CHROMOSOME $CHROMOSOME"
   echo "################################################################################"

   #
   # Input files
   #
   PEDIGREEFILE="chr"$CHROMOSOME"."$PEDIGREE_FILE_EXTENSION
   MAPFILE="map"$CHROMOSOME".dat"

   #
   # If files exists...
   #
   if [ -f $PEDIGREEFILE ] && [ -f $MAPFILE ]
   then

      #
      # 'dos2unix'
      #
      #if [ $MERLIN_WIPED -eq 0 ]
      #then
      #   convert_to_unix_format
      #fi

      #
      # Count number of markers from pedigree file
      #
      if [ $DICHOTOMOUS_TRAIT -eq 1 ]
      then
         NUMBER_OF_LOCI=`awk 'END {print (NF - 6)/2}' $PEDIGREEFILE`
      else
         NUMBER_OF_LOCI=`awk 'END {print (NF - 5)/2}' $PEDIGREEFILE`
      fi

      #
      # Chromosome directory
      #
	   CHROMOSOMEDIR="chr"$CHROMOSOME
      if [ ! -d $CHROMOSOMEDIR ]
      then
         mkdir $CHROMOSOMEDIR
      fi


	   #
      # Output files
      #
      if [ $MALES -eq 1 ]
      then
         MERLIN_MULTIPOINT_OUT="merlin_multipoint_"$CHROMOSOME"_males.out"
         MERLIN_MULTIPOINT_PDF="merlin_multipoint_"$CHROMOSOME"_males"
         MERLIN_SINGLEPOINT_OUT="merlin_singlepoint_"$CHROMOSOME"_males.out"
         MERLIN_SINGLEPOINT_PDF="merlin_singlepoint_"$CHROMOSOME"_males"
      elif [ $FEMALES -eq 1 ]
	   then
         MERLIN_MULTIPOINT_OUT="merlin_multipoint_"$CHROMOSOME"_females.out"
         MERLIN_MULTIPOINT_PDF="merlin_multipoint_"$CHROMOSOME"_females"
         MERLIN_SINGLEPOINT_OUT="merlin_singlepoint_"$CHROMOSOME"_females.out"
         MERLIN_SINGLEPOINT_PDF="merlin_singlepoint_"$CHROMOSOME"_females"
      else
         MERLIN_MULTIPOINT_OUT="merlin_multipoint_"$CHROMOSOME".out"
         MERLIN_MULTIPOINT_PDF="merlin_multipoint_"$CHROMOSOME".pdf"
         MERLIN_SINGLEPOINT_OUT="merlin_singlepoint_"$CHROMOSOME".out"
         MERLIN_SINGLEPOINT_PDF="merlin_singlepoint_"$CHROMOSOME".pdf"
      fi

      MERLIN_OVERRIDE_OUTPUT_FILE="merlin_override_"$CHROMOSOME".out"
      MERLIN_ERROR_OUTPUT_FILE="merlin_error_"$CHROMOSOME".out"

      if [ $MERLIN_WIPED -eq 0 ]
      then
         echo "--------------------------------------------------------------------------------"
         echo " NUMBER OF LOCI $NUMBER_OF_LOCI "
         echo "--------------------------------------------------------------------------------"
      fi
      
	   if [ -d $CHROMOSOMEDIR ]
	   then
         #
         # Change to chromosome directory
         #
	      cd $CHROMOSOMEDIR
         #
	      # PedConvert
         #
         if [ $MERLIN_NPL -eq 1 ] || [ $MERLIN_MODEL -eq 1 ]
         then
            PEDCONVERT_EXTRA_OPTIONS=$PEDCONVERT_EXTRA_OPTIONS" --discrete "
         fi
         if [ $MERLIN_MODEL -eq 1 ]
         then
            PEDCONVERT_EXTRA_OPTIONS=$PEDCONVERT_EXTRA_OPTIONS" -i ../"$MODEL_FILE
         fi
         if [ $COVARIATES -eq 0 ]
         then
            PEDCONVERT_EXTRA_OPTIONS=$PEDCONVERT_EXTRA_OPTIONS" --all "
         fi
         if [ $DISABLE_MAP_SET -eq 1 ]
         then
            PEDCONVERT_EXTRA_OPTIONS=$PEDCONVERT_EXTRA_OPTIONS" --dm "$DISABLE_MAP_POSITION
         fi
         if [ $MERLIN_WIPED -eq 0 ]
         then
            $PEDCONVERT_COMMAND -p ../$PEDIGREEFILE -f ../$PHENOTYPE_FILE  -m ../$MAPFILE -n $NUMBER_OF_LOCI $PEDCONVERT_EXTRA_OPTIONS 
         fi
         #
         # Mendelian check
         # 
         if [ $PEDCHECK_SKIP -eq 0 ]
         then
            mendelian_check
         fi
         if [ $MENDELIAN_ERRORS_FOUND -eq 0 ]
         then        
            #
            # Merlin input files
            #
            if [ $MERLIN_WIPED -eq 1 ]
            then
               echo "################################################################################"
               echo "# Pedwipe"
               echo "################################################################################"
               if [ -f merlin.err ]
               then
                  $PEDWIPE_COMMAND
               else
                  echo "Error: merlin.err file not found! Is error analysis done?"
               fi
               MERLIN_PEDIGREEFILE="wiped.ped"
               MERLIN_LOCUSFILE="wiped.dat"
            else
               MERLIN_PEDIGREEFILE="merlin.ped"
               MERLIN_LOCUSFILE="merlin.dat"
            fi
            MERLIN_MAPFILE="merlin.map"

            #
            # CHROMOSOME-X
            #
            if [ $CHROMOSOME -eq 23 ]
            then
               MERLIN_COMMAND=$XMERLIN_COMMAND
               PEDSTATS_OPTIONS=$PEDSTATS_OPTIONS" --chromosomeX "
            fi
            
            #
            # Chromosome greater than 23
            #
            if [ $CHROMOSOME -gt 23 ]
            then
               MERLIN_COMMAND="merlin"
               PEDSTATS_OPTIONS=" --hardyWeinberg --checkAll --showAll "
            fi

            #
            #
            #
            if [ $MAKE_FILES_ONLY -eq 0 ]
            then
               #
               # Pedstats analysis
               #
               if [ $PEDSTATS -eq 1 ]
               then
                  #
                  # Pedstats
                  #
                  echo "################################################################################"
                  echo "# Pedstats"
                  echo "################################################################################"
                  PEDSTAST_OUPUT_FILE="pedstats_"$CHROMOSOME".out"
                  $PEDSTATS_COMMAND -p $MERLIN_PEDIGREEFILE -d $MERLIN_LOCUSFILE $PEDSTATS_OPTIONS -x $MERLIN_MISSING | tee $PEDSTAST_OUPUT_FILE
                  if [ -f $PEDSTAST_OUPUT_FILE ]
                  then
                     mv -v $PEDSTAST_OUPUT_FILE ../$RESULTS_FOLDER
                  fi
               elif [ $MERLIN_OVERRIDE -eq 1 ]
               then
                  #
                  # Merlin OVERRIDE
                  #
                  echo "################################################################################"
                  echo "# Merlin (OVERRIDE MODE)"
                  echo "################################################################################"
                  $MERLIN_COMMAND $MERLIN_OPTIONS | tee $MERLIN_OVERRIDE_OUTPUT_FILE
               elif [ $MERLIN_ERROR -eq 1 ]
               then
                  #
                  # Merlin ERROR
                  #
                  echo "################################################################################"
                  echo "# Merlin (GENOTYPING ERROR ANALYSIS)"
                  echo "################################################################################"
                  $MERLIN_COMMAND -p $MERLIN_PEDIGREEFILE -d $MERLIN_LOCUSFILE -m $MERLIN_MAPFILE -x $MERLIN_MISSING --markerNames --quiet --error --bits $MERLIN_BITS > $MERLIN_ERROR_OUTPUT_FILE
               elif [ $MERLIN_SIMULATE -eq 1 ]
               then
                  #
                  # Merlin SIMULATE
                  #
                  echo "################################################################################"
                  echo "# Merlin (SIMULATE) -  $MERLIN_REPLICATES replicates in this simulation"
                  echo "################################################################################"
                  MERLIN_SIMULATE_OUTPUT_FILE="merlin_simulate_"$CHROMOSOME".out"
                  if [ -f $MERLIN_SIMULATE_OUTPUT_FILE ]
                  then
                     rm -f $MERLIN_SIMULATE_OUTPUT_FILE
                  fi
                  for ((REPLICATE=1 ; REPLICATE <=$MERLIN_REPLICATES ; REPLICATE++))
                  do
                     RANDOMSEED=$RANDOM
                     echo "Replicate # $REPLICATE - Seed $RANDOMSEED"
                     $MERLIN_COMMAND -p $MERLIN_PEDIGREEFILE -d $MERLIN_LOCUSFILE -m $MERLIN_MAPFILE -r $RANDOMSEED -x $MERLIN_MISSING --simulate --save --bits $MERLIN_BITS >> $MERLIN_SIMULATE_OUTPUT_FILE
                     MERLIN_SIMULATE_REPLICATE_OUTPUT_FILE="merlin_simulate_"$CHROMOSOME".out"
                     if [ $MERLIN_ANALYSIS -eq 0 ] || [ $MERLIN_ANALYSIS -eq 1 ]
                     then
                        echo "Singlepoint analysis..."
                        $MERLIN_COMMAND -p merlin-replicate.ped -d merlin-replicate.dat -m $MERLIN_MAPFILE $MERLIN_OPTIONS --singlepoint > $MERLIN_SINGLEPOINT_OUT"."$REPLICATE
                        mv -v $MERLIN_SINGLEPOINT_OUT"."$REPLICATE ../$RESULTS_FOLDER
                     fi
                     #
                     if [ $MERLIN_ANALYSIS -eq 0 ] || [ $MERLIN_ANALYSIS -eq 2 ]
                     then
                        echo "Multipoint analysis..."
                        $MERLIN_COMMAND -p merlin-replicate.ped -d merlin-replicate.dat -m $MERLIN_MAPFILE $MERLIN_OPTIONS > $MERLIN_MULTIPOINT_OUT"."$REPLICATE
                        mv -v $MERLIN_MULTIPOINT_OUT"."$REPLICATE ../$RESULTS_FOLDER
                     fi                  
                     rm -f merlin-replicate.ped merlin-replicate.dat merlin-replicate.map merlin-replicate.freq
                  done
               else
                  #
                  # Merlin analysis
                  #
                  echo "################################################################################"
                  echo "# Merlin (ANALYSIS)"
                  echo "################################################################################"
                  if [ -f $MERLIN_PEDIGREEFILE ] && [ -f $MERLIN_LOCUSFILE ] && [ -f $MERLIN_MAPFILE ]
                  then
                     if [ $MERLIN_ANALYSIS -eq 0 ] || [ $MERLIN_ANALYSIS -eq 1 ]
                     then
                        echo "Singlepoint analysis...(Merlin options used: $MERLIN_OPTIONS --singlepoint)"
                        $MERLIN_COMMAND -p $MERLIN_PEDIGREEFILE -d $MERLIN_LOCUSFILE -m $MERLIN_MAPFILE $MERLIN_OPTIONS --singlepoint > $MERLIN_SINGLEPOINT_OUT
                        if [ $MERLIN_PDF -eq 1 ]
                        then
                           mv merlin.pdf $MERLIN_SINGLEPOINT_PDF
                        fi
                     fi
                     #
                     if [ $MERLIN_ANALYSIS -eq 0 ] || [ $MERLIN_ANALYSIS -eq 2 ]
                     then
                        echo "Multipoint analysis...(Merlin options used: $MERLIN_OPTIONS)"
                        $MERLIN_COMMAND -p $MERLIN_PEDIGREEFILE -d $MERLIN_LOCUSFILE -m $MERLIN_MAPFILE $MERLIN_OPTIONS > $MERLIN_MULTIPOINT_OUT
                        if [ $MERLIN_PDF -eq 1 ]
                        then
                           mv merlin.pdf $MERLIN_MULTIPOINT_PDF
                        fi
                     fi
                  else
                     echo "Warning: Input files are missing: $MERLIN_PEDIGREEFILE, $MERLIN_LOCUSFILE, $MERLIN_MAPFILE"
                  fi
                  #
                  # Move results
                  #
                  for OUTPUTFILE in $MERLIN_SINGLEPOINT_OUT $MERLIN_SINGLEPOINT_PDF $MERLIN_MULTIPOINT_OUT $MERLIN_MULTIPOINT_PDF
                  do
                     if [ -f $OUTPUTFILE ]
                     then
                        mv -v $OUTPUTFILE ../$RESULTS_FOLDER
                     fi
                  done
               fi
            fi
         else
            echo "--------------------------------------------------------------------------------"
            echo " Mendelian error(s) found, see pedcheck.err, skipping this chromosome..."
            echo "--------------------------------------------------------------------------------"                  
         fi
	      cd ..
	   fi
   else
      echo " Skipping...no pedigree file $PEDIGREEFILE and/or map file $MAPFILE found! "
   fi
done

echo
echo "End of the AutoMerlin!"
echo
