#!/bin/bash
################################################################################
################################################################################
#
# Program : AutoQualitative.sh - Automated genome-wide scan with Analyze
# Version : 1.0.3
# Date    : 18.10.2006
# Author  : Tero Hiekkalinna
# Email   : Tero Hiekkalinna@Helsinki.FI (tero hiekkalinna@ktl.fi)
#
# COPYRIGHT (C) Tero Hiekkalinna 2003-2006
#
################################################################################
################################################################################

#
# Variables
#
ANALYZE_PREP_COMMAND="analprep"
ANALYZE_COMMAND="analyze"
ANALYZE_COMMAND_FILE="analyze.dat"
ANALYSIS_ANALYSIS=0
ANALYZE_OPTION_FILE="analyze.dat"
RESULTS_FOLDER="all_results"
PARAMETRIC_LINKAGE_ANALYSIS=0
NUCLEAR_PEDIGREE_LINKAGE_ANALYSIS=0
SIBPAIR_LINKAGE_ANALYSIS=0
HRR_ANALYSIS=0
TDT_ANALYSIS=0
PEDCONVERT_COMMAND="pedconvert"
PEDCONVERT_EXTRA_OPTIONS=" --prog analyze --discrete "
DOWNCODE_COMMAND="downcode"
DOWNCODE_OPTIONS=" --prog analyze "
MODEL_FILE="model.dat"
LOOP_FILE="loops.dat"
LOOP_FILE_SET=0
MAKEPED_COMMAND="makeped"
MAKEPED_OPTIONS_FILE="makeped.dat"
MALES=0
FEMALES=0
MAKE_FILES_ONLY=0
START_CHROMOSOME=1
STOP_CHROMOSOME=22
PEDIGREE_FILE_EXTENSION="raw"
PHENOTYPE_FILE="phen.dat"
PEDCHECK_COMMAND="pedcheck"
PEDCHECK_OPTIONS=""
PEDCHECK_LEVEL=2
PEDCHECK_NAMES_FILE="names.dat"
PEDCHECK_SKIP=0
MENDELIAN_ERRORS_FOUND=0
PLOT_DATA_ALL_FILE="plotdata_all.out"
POSTSCRIPT=0
DEBUG=0
VERBOSE=0

#
# Program info & copyright notice
#
function info(){
   echo "################################################################################"
   echo "# Program : AutoQualitative.sh - Automated genome-wide scan with Analyze       #"
   echo "# Version : 1.0.3                                                              #"
   echo "# Date    : 18.10.2006                                                         #"
   echo "# Author  : Tero Hiekkalinna                                                   #"
   echo "# Email   : Tero Hiekkalinna@Helsinki.FI (tero hiekkalinna@ktl.fi)             #"
   echo "#                                                                              #"
   echo "# COPYRIGHT (C) Tero Hiekkalinna 2003-2006                                     #"
   echo "################################################################################"
}

#
# Usage
#
function usage(){
   info
   analysis_options
   echo
   echo "Usage: autoqualitative.sh [OPTION]..."
   echo
   echo "Analysis options:"
   echo "================="
   echo
   echo " --linkage"
   echo "    Parametric linkage analysis (and heterogeneity testing)"
   echo " --nuclear"
   echo "    Nuclear pedigree linkage analysis (and heterogeneity testing)"
   echo " --sibpair"
   echo "    Sibpair linkage analysis"
   echo " --hrr"
   echo "    Haplotype-based haplotype relative risk test (HHRR)"
   echo " --tdt"
   echo "    Transmission Disequilibrium Test"
   echo " --all"
   echo "    All analysis above"
   echo
   echo " --chr [start] [stop]"
   echo "    Specify range of chromosomes to be analysed. Chromosome X is 23 and" 
   echo "    default range is 1-22"
   #echo " --analysis [single,multi]"
   #echo "    Specify single or multipoint analysis only, default is both"
   echo " --only [males,females]"
   echo "    Sex specific analysis, phenotypes of females are set to unknown and" 
   echo "    vice versa"
   echo " --pedcheck-cmd [name]"
   echo "    Specify pedcheck command (Default is $PEDCHECK_COMMAND)"
   echo " --skip-pedcheck"
   echo "    Skip Mendelian check (usefull if genotype data is known to be OK and"
   echo "    pedigrees are big!)"
   echo " --pedcheck-level [level]"
   echo "    Specify PedCheck check level (Default is $PEDCHECK_LEVEL)"
   echo
   echo "Input/Output file control options:"
   echo "=================================="
   echo
   echo " --phen [file]"
   echo "    Specify your own phenotype file name. Default is $PHENOTYPE_FILE"
   echo " --model [file]"
   echo "    Specify your own model file name. Default is $MODEL_FILE"
   echo " --loop [file]"
   echo "    Specify your loop file name, which will be used for breaking with MAKEPED"
   echo "    program."
   echo " --ignore"
   echo "    Pedigree file includes discrete trait as a first phenotype. It will be"
   echo "    read and ignored"
   echo " --liab"
   echo "    Phenotype file contains liability class after trait phenotype"
   echo " --ext [extension]"
   echo "    Pedigree file extension, example: raw or pre. Default is raw"
   echo " --makefilesonly"
   echo "    Make only Analyze input files. Does not run analysis."
   echo
   echo "Help:"
   echo "====="
   echo
   echo " --help or -h"
   echo "    Print this help. See AUTOGSCAN for full documentation."
   echo
}

#
# debug
#
function debug(){
   echo "################################################################################"
   echo "PEDCONVERT_EXTRA_OPTIONS=$PEDCONVERT_EXTRA_OPTIONS"
   echo "MALES=$MALES"
   echo "FEMALES=$FEMALES"
   echo "MAKE_FILES_ONLY=$MAKE_FILES_ONLY"
   echo "PEDIGREE_FILE_EXTENSION=$PEDIGREE_FILE_EXTENSION"
   echo "START_CHROMOSOME=$START_CHROMOSOME"
   echo "STOP_CHROMOSOME=$STOP_CHROMOSOME"
   echo "PHENOTYPE_FILE=$PHENOTYPE_FILE"
   echo "DEBUG=$DEBUG"
   echo "VERBOSE=$VERBOSE"
   echo "################################################################################"
   exit
}

#
# Prints used options
#
function analysis_options(){
   echo "################################################################################"
   echo "# Default options are: "
   echo "# Results folder: `pwd`/$RESULTS_FOLDER "
   echo "################################################################################"
}

#
# Convert input files to Unix format
#
function convert_to_unix_format(){
   dos2unix -q -V >& /dev/null
   echo "--------------------------------------------------------------------------------"
   if [ $? != '0' ]
   then
      echo " Warning: Program dos2unix not found!"
   else
      echo " Converting input files to Unix format!"
      for INPUTFILE in $PEDIGREEFILE $PHENOTYPE_FILE $MAPFILE
      do
         dos2unix -q $INPUTFILE
      done
   fi
   echo "--------------------------------------------------------------------------------"
}

#
# Mendelian check
#
function mendelian_check(){
   echo "--------------------------------------------------------------------------------"
   echo " PedCheck "
   echo "--------------------------------------------------------------------------------"
   if [ -f pedcheck.err ]
   then
      mv pedcheck.err pedcheck.err.old
   fi
   MENDELIAN_ERRORS_FOUND=0
   if [ $CHROMOSOME -eq 23 ]
   then
      PEDCHECK_OPTIONS=$PEDCHECK_OPTIONS" -x "
   fi
   $PEDCHECK_COMMAND -n $PEDCHECK_NAMES_FILE -p ../$PEDIGREEFILE -$PEDCHECK_LEVEL $PEDCHECK_OPTIONS >& /dev/null
   if [ -f pedcheck.err ]
   then
      GENOTYPE_ERROR_COUNT=`grep -c "GENOTYPE ERROR" pedcheck.err`
      if [ $GENOTYPE_ERROR_COUNT -ne 0 ]
      then
         MENDELIAN_ERRORS_FOUND=1
      else
         echo "Mendelian check passed (level $PEDCHECK_LEVEL)..."
      fi
   else
      echo "Warning: pedcheck.err file not found, is pedcheck installed?"
   fi
}

#
# Plotting (Experimental!!)
#
function plot_results(){
   echo "Generating graphical output using Gnuplot...(Experimental!)"
   cd $RESULTS_FOLDER
   rm -f plotdata_all.out
   for ((CHROMOSOME=$START_CHROMOSOME ; CHROMOSOME <=$STOP_CHROMOSOME  ; CHROMOSOME++))
   do
      PLOTDATAFILE="plotdata_"$CHROMOSOME".out"
      cat $PLOTDATAFILE >> plotdata_all.out
   done
   ags_genomewideplot.sh
   cd ..
}

#
# MAIN
#

#
# Parse command line arguments
#
while [ $# -ge 1 ]; do
   case "$1" in
      --linkage)
         PARAMETRIC_LINKAGE_ANALYSIS=1
         ;;
      --nuclear)
         NUCLEAR_PEDIGREE_LINKAGE_ANALYSIS=1
         ;;
      --sibpair)
         SIBPAIR_LINKAGE_ANALYSIS=1
         ;;
      --hrr)
         HRR_ANALYSIS=1
         ;;
      --tdt)
         TDT_ANALYSIS=1
         ;;
      --all)
         PARAMETRIC_LINKAGE_ANALYSIS=1
         NUCLEAR_PEDIGREE_LINKAGE_ANALYSIS=1
         SIBPAIR_LINKAGE_ANALYSIS=1
         HRR_ANALYSIS=1
         TDT_ANALYSIS=1
         ;;
      --only)
         if [ $2 == "males" ]
         then
            MALES=1
            PEDCONVERT_EXTRA_OPTIONS=$PEDCONVERT_EXTRA_OPTIONS" --only males "
         elif [ $2 == "females" ]
         then
            FEMALES=1
            PEDCONVERT_EXTRA_OPTIONS=$PEDCONVERT_EXTRA_OPTIONS" --only females "
         else
            usage
            echo
            echo "Error: argument after --only must be males or females!"
            echo
            exit 0
         fi
         shift
         ;;
      --makefilesonly)
         MAKE_FILES_ONLY=1
         ;;
      --ignore)
         DICHOTOMOUS_TRAIT=1
         PEDCONVERT_EXTRA_OPTIONS=$PEDCONVERT_EXTRA_OPTIONS" --ignore "
         ;;
      --ext)
         PEDIGREE_FILE_EXTENSION=$2
         shift
         ;;
      --chr)
         START_CHROMOSOME=$2
         STOP_CHROMOSOME=$3
         if [ $START_CHROMOSOME -lt 1 ] || [ $START_CHROMOSOME -gt 23 ]
         then
            usage
            echo
            echo "Error: Illegal start chromosome number: $START_CHROMOSOME"
            echo
            exit 1
         elif [ $STOP_CHROMOSOME -lt 1 ] || [ $STOP_CHROMOSOME -gt 23 ]
         then
            usage
            echo
            echo "Error: Illegal stop chromosome number: $STOP_CHROMOSOME"
            echo
            exit 1
         elif [ $START_CHROMOSOME -gt $STOP_CHROMOSOME ]
         then
            usage
            echo
            echo "Error: Start chromosome $START_CHROMOSOME is greater than stop chromosome $STOP_CHROMOSOME!"
            echo
            exit 1
         fi
         shift; shift
         ;;
      --phen)
         if [ -f $2 ]
         then
            PHENOTYPE_FILE=$2
         else
            info
            echo
            echo "Error: Phenotype file $2 not found!"
            echo
            exit 1
         fi
         shift
         ;;
       --model)
         if [ -f $2 ]
         then
            MODEL_FILE=$2
         else
            info
            echo
            echo "Error: Model file $2 not found!"
            echo
            exit 1
         fi
         shift
         ;;
       --loop)
         if [ -f $2 ]
         then
            LOOP_FILE_SET=1
            LOOP_FILE=$2
            echo "y" > $MAKEPED_OPTIONS_FILE
            echo "y" >> $MAKEPED_OPTIONS_FILE
            echo "../$LOOP_FILE" >> $MAKEPED_OPTIONS_FILE
            echo "y" >> $MAKEPED_OPTIONS_FILE
            echo "" >> $MAKEPED_OPTIONS_FILE
         else
            info
            echo
            echo "Error: Loop file $2 not found!"
            echo
            exit 1
         fi
         shift
         ;;
      --analysis)
         if [ $2 == "single" ]
         then
            ANALYSIS_ANALYSIS=1
         elif [ $2 == "multi" ]
         then
            ANALYSIS_ANALYSIS=2
         else
            usage
            echo
            echo "Error: argument after --analysis must be single or multi!"
            echo
            exit 1
         fi
         shift
         ;;
      --liab)
         DOWNCODE_OPTIONS=$DOWNCODE_OPTIONS" --liab "
         ;;
      --skip-pedcheck)
         PEDCHECK_SKIP=1
         ;;
      --pedcheck-level)
         PEDCHECK_LEVEL=$2
         shift
         ;;
      --ps)
         POSTSCRIPT=1
         ;;
      --debug)
         DEBUG=1
         ;;
      --verbose)
         VERBOSE=1
         ;;
      --help|-h)
         usage
         exit 1
         ;;
      *)
         usage
         echo "Error: Unknown option:" $1
         exit 1
         ;;
   esac
   shift
done

info
analysis_options

#
# verbose debug
#
if [ $DEBUG -eq 1 ]
then
   debug
fi

#
# Create results folder
#
if [ ! -d $RESULTS_FOLDER ]
then
   mkdir $RESULTS_FOLDER
fi

#
# Chromosomes...
#
for ((CHROMOSOME=$START_CHROMOSOME ; CHROMOSOME <=$STOP_CHROMOSOME  ; CHROMOSOME++))
do

   echo "################################################################################"
   echo "# CHROMOSOME $CHROMOSOME"
   echo "################################################################################"

   #
   # Input files
   #
   PEDIGREEFILE="chr"$CHROMOSOME"."$PEDIGREE_FILE_EXTENSION
   MAPFILE="map"$CHROMOSOME".dat"

   #
   # CHROMOSOME-X
   #
   if [ $CHROMOSOME -eq 23 ]
   then
      MODEL_FILE="xmodel.dat"
      DOWNCODE_OPTIONS=$DOWNCODE_OPTIONS" --x "
   fi

   #
   # If pedigree file exists, start analysis
   #
   if [ -f $PEDIGREEFILE ] && [ -f $MAPFILE ] && [ -f $MODEL_FILE ] && [ -f $PHENOTYPE_FILE ]
   then

      #
      # Count number of markers from pedigree file
      #
      if [ $DICHOTOMOUS_TRAIT -eq 1 ]
      then
         NUMBER_OF_LOCI=`awk 'END {print (NF - 6)/2}' $PEDIGREEFILE`
      else
         NUMBER_OF_LOCI=`awk 'END {print (NF - 5)/2}' $PEDIGREEFILE`
      fi

      #
      # Chromosome directory
      #
	   CHROMOSOMEDIR="chr"$CHROMOSOME
      if [ ! -d $CHROMOSOMEDIR ]
      then
         mkdir $CHROMOSOMEDIR
      fi

	   #
      # Output files
      #
      if [ $MALES -eq 1 ]
      then
         ANALYZE_SUMMARY_OUT="analyze_summary_"$CHROMOSOME"_males.out"
         ANALYZE_FINAL_OUT="analyze_final_"$CHROMOSOME"_males.out"
         ANALYZE_POSTSCRIPT="analyze_plot_"$CHROMOSOME"_males.ps"
      elif [ $FEMALES -eq 1 ]
	   then
         ANALYZE_SUMMARY_OUT="analyze_summary_"$CHROMOSOME"_females.out"
         ANALYZE_FINAL_OUT="analyze_final_"$CHROMOSOME"_females.out"
         ANALYZE_POSTSCRIPT="analyze_plot_"$CHROMOSOME"_females.ps"
      else
         ANALYZE_SUMMARY_OUT="analyze_summary_"$CHROMOSOME".out"
         ANALYZE_FINAL_OUT="analyze_final_"$CHROMOSOME".out"
         ANALYZE_POSTSCRIPT="analyze_plot_"$CHROMOSOME".ps"
      fi

      echo "--------------------------------------------------------------------------------"
      echo " NUMBER OF LOCI $NUMBER_OF_LOCI "
      echo "--------------------------------------------------------------------------------"

	   if [ -d $CHROMOSOMEDIR ]
	   then
         #
         # Change to chromosome directory
         #
	      cd $CHROMOSOMEDIR

         #
         # PedConvert
         #
         $PEDCONVERT_COMMAND -p ../$PEDIGREEFILE -f ../$PHENOTYPE_FILE  -m ../$MAPFILE -n $NUMBER_OF_LOCI $PEDCONVERT_EXTRA_OPTIONS 

         #
         # Mendelian check
         # 
         if [ $PEDCHECK_SKIP -eq 0 ]
         then
            mendelian_check
         fi
         if [ $MENDELIAN_ERRORS_FOUND -eq 0 ]
         then        
            #
            # DownCode
            #
            PEDIGREEFILE="analyze.ped"
            $DOWNCODE_COMMAND $DOWNCODE_OPTIONS -p $PEDIGREEFILE -map ../$MAPFILE -m ../$MODEL_FILE -n $NUMBER_OF_LOCI --trait --pre 
            LOCUSFILE="datain.dat" 
            PEDIGREEFILE="analyze_coded.ped"                    
            
            #
            # Makeped
            #  
            if [ $LOOP_FILE_SET -eq 0 ]
            then
               $MAKEPED_COMMAND $PEDIGREEFILE pedin.dat n >& /dev/null
            else
               $MAKEPED_COMMAND $PEDIGREEFILE pedin.dat < ../$MAKEPED_OPTIONS_FILE >& /dev/null
            fi
            PEDIGREEFILE="pedin.dat"

            #
            # Command file
            #
            echo $PARAMETRIC_LINKAGE_ANALYSIS > $ANALYZE_OPTION_FILE
            echo $HRR_ANALYSIS >> $ANALYZE_OPTION_FILE
            echo $SIBPAIR_LINKAGE_ANALYSIS >> $ANALYZE_OPTION_FILE
            echo $TDT_ANALYSIS >> $ANALYZE_OPTION_FILE
            echo $NUCLEAR_PEDIGREE_LINKAGE_ANALYSIS >> $ANALYZE_OPTION_FILE
            echo "0" >> $ANALYZE_OPTION_FILE
            echo "0" >> $ANALYZE_OPTION_FILE
            echo "" >> $ANALYZE_OPTION_FILE

            #
            # Analyze
            #
            if [ $MAKE_FILES_ONLY -eq 0 ]
            then
               echo "################################################################################"
               echo "# ANALYZE "
               echo "################################################################################"
               if [ -f $PEDIGREEFILE ] && [ -f $LOCUSFILE ]
               then
                  echo "Analyzing..."
                  if [ $VERBOSE -eq 0 ]
                  then
                     $ANALYZE_PREP_COMMAND < $ANALYZE_OPTION_FILE >& /dev/null
                     sh $ANALYZE_COMMAND >& /dev/null
                  else
                     $ANALYZE_PREP_COMMAND < $ANALYZE_OPTION_FILE
                     sh $ANALYZE_COMMAND
                  fi
                  
                  #
                  # New summary (more informative)
                  #
                  scavenge -d database.out -m ../$MAPFILE

                  if [ $POSTSCRIPT -eq 1 ]
                  then
                     PLOTDATAFILE="plotdata_"$CHROMOSOME".out"
                     mv -v plotdata.out $PLOTDATAFILE
                  fi

                  mv summary.out $ANALYZE_SUMMARY_OUT
                  mv analyze.final $ANALYZE_FINAL_OUT
                  echo "...done"
               else
                  echo "Warning: Input files are missing: $PEDIGREEFILE and/or $LOCUSFILE"
               fi
               #
               # Move results
               #
               echo
               echo "Moving results files if any..."
               echo
               for OUTPUTFILE in $ANALYZE_SUMMARY_OUT $ANALYZE_FINAL_OUT $ANALYZE_POSTSCRIPT $PLOTDATAFILE
               do
                  if [ -f $OUTPUTFILE ]
                  then
                     mv -v $OUTPUTFILE ../$RESULTS_FOLDER
                  fi
               done
               
               #
               # Remove files
               #
               echo
               rm -rf FASTLINK.err
            fi
         else
            echo "--------------------------------------------------------------------------------"
            echo " Mendelian error(s) found, see pedcheck.err, skipping this chromosome..."
            echo "--------------------------------------------------------------------------------"         
         fi
         cd ..     
      fi
   else
      echo " Skipping...no pedigree file $PEDIGREEFILE or map file $MAPFILE or model file $MODEL_FILE or phenotype file $PHENOTYPE_FILE found! "
   fi
done

if [ $POSTSCRIPT -eq 1 ]
then
   plot_results
fi

echo
echo "End of the AutoQualitative!"
echo
