#!/bin/bash
################################################################################
################################################################################
#
# Program : AutoSolar.sh - Automated genome-wide scan with Solar
# Version : 1.0.3
# Date    : 18.10.2006
# Author  : Tero Hiekkalinna
# Email   : Tero Hiekkalinna@Helsinki.FI (tero hiekkalinna@ktl.fi)
#
# COPYRIGHT (C) Tero Hiekkalinna 2003-2006
#
################################################################################
################################################################################

#
# Variables
#
SOLAR_COMMAND="solar"
SOLAR_ALL_USER_COMMAND_USED=0
SOLAR_USER_COMMAND_FILE="solar.dat"
SOLAR_COMMAND_FILE="solar.in"
PEDCONVERT_COMMAND="pedconvert"
PEDCONVERT_OPTIONS=" --all --prog solar "
DOWNCODE_COMMAND="downcode"
DOWNCODE_OPTIONS=" --prog solar "
MALES=0
FEMALES=0
MAKE_FILES_ONLY=0
START_CHROMOSOME=1
STOP_CHROMOSOME=22
PEDIGREE_FILE_EXTENSION="raw"
PHENOTYPE_FILE="phen.dat"
SOLAR_ANALYSIS=0
SOLAR_OUTDIR="results"
IBD_DIR=`pwd`"/ibd"
MIBD_DIR=`pwd`"/mibd"
CALC_SOLAR_IBDS=0
CALC_SOLAR_IBDS_ONLY=0
POLYGENIC_ONLY=0
INTERVAL=5
FINEMAP=-1
PEDCHECK_COMMAND="pedcheck"
PEDCHECK_OPTIONS=""
PEDCHECK_LEVEL=2
PEDCHECK_NAMES_FILE="names.dat"
PEDCHECK_SKIP=0
MENDELIAN_ERRORS_FOUND=0
DEBUG=0
VERBOSE=0
RESULTS_FOLDER="all_results"

#
# Program info & copyright notice
#
function info(){
   echo "################################################################################"
   echo "# Program : AutoSolar.sh - Automated genome-wide scan with Solar               #"
   echo "# Version : 1.0.3                                                              #"
   echo "# Date    : 18.10.2006                                                         #"
   echo "# Author  : Tero Hiekkalinna                                                   #"
   echo "# Email   : Tero Hiekkalinna@Helsinki.FI (tero hiekkalinna@ktl.fi)             #"
   echo "#                                                                              #"
   echo "# COPYRIGHT (C) Tero Hiekkalinna 2003-2005                                     #"
   echo "################################################################################"
}

#
# Usage
#
function usage(){
   info
   analysis_options
   echo
   echo "Usage: autosolar.sh [OPTION]..."
   echo
   echo "Analysis options:"
   echo "================="
   echo 
   echo " --solar-cmd [name]"
   echo "    Specify solar command (Default is $SOLAR_COMMAND)"
   echo " --solar-file [file]"
   echo "    Specify solar analysis option file. Default is $SOLAR_USER_COMMAND_FILE"
   echo " --ibd"
   echo "    Calculates IBDs and MIBDs before scan"
   echo " --ibd-only"
   echo "    Calculates only IBDs and MIBDs, no linkage scan is done."
   echo " --ibd-dir [full path to IBD directory]"
   echo "    Specify IBD directory where pre-calculated Solar IBD files are located"
   echo " --mibd-dir [full path to MIBD directory]"
   echo "    Specify MIBD directory where pre-calculated Solar MIBD files are located"
   echo " --polygenic-only"
   echo "    Polygenic screening only, no IBD/MIBD or linkage scan done."  
   echo " --interval [cM]"
   echo "    Specify multipoint mapping interval in cM (Default is $INTERVAL)"
   echo " --finemap [lod]"
   echo "    Specify lod score treshold for finemapping (This is disabled by default,"
   echo "    but specifying finemap lod will enable it)"
   echo " --chr [start] [stop]"
   echo "    Specify range of chromosomes to be analysed. Chromosome X is 23 and"
   echo "    default range is 1-22"
   echo " --analysis [single,multi]"
   echo "    Specify single or multipoint analysis only, default is both"
   echo " --only [males,females]"
   echo "    Sex specific analysis, phenotypes of females are set to unknown and"
   echo "    vice versa"
   echo " --pedcheck-cmd [name]"
   echo "    Specify pedcheck command (Default is $PEDCHECK_COMMAND)"
   echo " --skip-pedcheck"
   echo "    Skip Mendelian check (usefull if genotype data is known to be OK and"
   echo "    pedigrees are big!)"
   echo " --pedcheck-level [level]"
   echo "    Specify PedCheck check level (Default is $PEDCHECK_LEVEL)"
   echo
   echo "Input/Output file control options:"
   echo "=================================="
   echo
   echo " --phen [file]"
   echo "    Specify your own phenotype file name. Default is phen.dat"
   echo " --ignore"
   echo "    Pedigree file includes discrete trait as a first phenotype. It will be"
   echo "    read and ignored"
   echo " --ext [extension]"
   echo "    Pedigree file extension, example: raw or pre. Default is raw"
   echo " --makefilesonly"
   echo "    Make only Solar input files. Does not run analysis."
   echo
   echo "Help:"
   echo "====="
   echo
   echo " --help or -h"
   echo "    Print this help. See AUTOGSCAN for full documentation."
   echo
}

#
# debug
#
function debug(){
   echo "################################################################################"
   echo "PEDCONVERT_COMMAND=$PEDCONVERT_COMMAND"
   echo "PEDCONVERT_OPTIONS=$PEDCONVERT_OPTIONS"
   echo "MALES=$MALES"
   echo "FEMALES=$FEMALES"
   echo "MAKE_FILES_ONLY=$MAKE_FILES_ONLY"
   echo "PEDIGREE_FILE_EXTENSION=$PEDIGREE_FILE_EXTENSION"
   echo "START_CHROMOSOME=$START_CHROMOSOME"
   echo "STOP_CHROMOSOME=$STOP_CHROMOSOME"
   echo "PHENOTYPE_FILE=$PHENOTYPE_FILE"
   echo "SOLAR_ANALYSIS=$SOLAR_ANALYSIS"
   echo "DEBUG=$DEBUG"
   echo "VERBOSE=$VERBOSE"
   echo "SOLAR_COMMAND=$SOLAR_COMMAND"
   echo "SOLAR_ALL_USER_COMMAND_USED=$SOLAR_ALL_USER_COMMAND_USED"
   echo "SOLAR_USER_COMMAND_FILE=$SOLAR_USER_COMMAND_FILE"
   echo "SOLAR_COMMAND_FILE=$SOLAR_COMMAND_FILE"
   echo "SOLAR_ANALYSIS=$SOLAR_ANALYSIS"
   echo "SOLAR_OUTDIR=$SOLAR_OUTDIR"
   echo "IBD_DIR=$IBD_DIR"
   echo "MIBD_DIR=$MIBD_DIR"
   echo "CALC_SOLAR_IBDS=$CALC_SOLAR_IBDS"
   echo "INTERVAL=$INTERVAL"
   echo "FINEMAP=$FINEMAP"
   echo "DOWNCODE_COMMAND=$DOWNCODE_COMMAND"
   echo "DOWNCODE_OPTIONS=$DOWNCODE_OPTIONS"
   echo "RESULTS_FOLDER=$RESULTS_FOLDER"
   echo "################################################################################"
   exit
}

#
# Prints used options
#
function analysis_options(){
   echo "################################################################################"
   echo "# Default options are: "
   echo "# Results folder: `pwd`/$RESULTS_FOLDER "
   echo "################################################################################"
}

#
# Convert input files to Unix format
#
function convert_to_unix_format(){
   dos2unix -q -V >& /dev/null
   echo "--------------------------------------------------------------------------------"
   if [ $? != '0' ]
   then
      echo " Warning: Program dos2unix not found!"
   else
      echo " Converting input files to Unix format!"
      for INPUTFILE in $PEDIGREEFILE $PHENOTYPE_FILE $MAPFILE
      do
         dos2unix -q $INPUTFILE
      done
   fi
   echo "--------------------------------------------------------------------------------"
}

#
# Mendelian check
#
function mendelian_check(){
   echo "--------------------------------------------------------------------------------"
   echo " PedCheck "
   echo "--------------------------------------------------------------------------------"
   if [ -f pedcheck.err ]
   then
      mv pedcheck.err pedcheck.err.old
   fi
   MENDELIAN_ERRORS_FOUND=0
   if [ $CHROMOSOME -eq 23 ]
   then
      PEDCHECK_OPTIONS=$PEDCHECK_OPTIONS" -x "
   fi
   $PEDCHECK_COMMAND -n $PEDCHECK_NAMES_FILE -p ../$PEDIGREEFILE -$PEDCHECK_LEVEL $PEDCHECK_OPTIONS >& /dev/null
   if [ -f pedcheck.err ]
   then
      GENOTYPE_ERROR_COUNT=`grep -c "GENOTYPE ERROR" pedcheck.err`
      if [ $GENOTYPE_ERROR_COUNT -ne 0 ]
      then
         MENDELIAN_ERRORS_FOUND=1
      else
         echo "Mendelian check passed (level $PEDCHECK_LEVEL)..."
      fi
   else
      echo "Warning: pedcheck.err file not found, is pedcheck installed?"
   fi
}

#
# MAIN
#

#
# Parse command line arguments
#
while [ $# -ge 1 ]; do
   case "$1" in
      --only)
         if [ $2 == "males" ]
         then
            MALES=1
            PEDCONVERT_OPTIONS=$PEDCONVERT_OPTIONS" --only males "
         elif [ $2 == "females" ]
         then
            FEMALES=1
            PEDCONVERT_OPTIONS=$PEDCONVERT_OPTIONS" --only females "
         else
            usage
            echo
            echo "Error: argument after --only must be males or females!"
            echo
            exit 0
         fi
         shift
         ;;
      --makefilesonly)
         MAKE_FILES_ONLY=1
         ;;
      --ignore)
         DICHOTOMOUS_TRAIT=1
         PEDCONVERT_OPTIONS=$PEDCONVERT_OPTIONS" --ignore "
         DOWNCODE_OPTIONS=$DOWNCODE_OPTIONS" --trait "
         ;;
      --ext)
         PEDIGREE_FILE_EXTENSION=$2
         shift
         ;;
      --solar-cmd)
         SOLAR_COMMAND=$2
         shift
         ;;
      --solar-file)
         if [ -f $2 ]
         then
            SOLAR_USER_COMMAND_FILE=$2
         else
            info
            echo
            echo "Error: Solar command file $2 not found!"
            echo
            exit 1
         fi
         shift
         ;;
      --solar-over)
         if [ -f $2 ]
         then
            SOLAR_ALL_USER_COMMAND_USED=1
            SOLAR_USER_COMMAND_FILE=$2
         else
            info
            echo
            echo "Error: Solar command file $2 not found!"
            echo
            exit 1
         fi
         shift
         ;;
      --chr)
         START_CHROMOSOME=$2
         STOP_CHROMOSOME=$3
         if [ $START_CHROMOSOME -lt 1 ] || [ $START_CHROMOSOME -gt 23 ]
         then
            usage
            echo
            echo "Error: Illegal start chromosome number: $START_CHROMOSOME"
            echo
            exit 1
         elif [ $STOP_CHROMOSOME -lt 1 ] || [ $STOP_CHROMOSOME -gt 23 ]
         then
            usage
            echo
            echo "Error: Illegal stop chromosome number: $STOP_CHROMOSOME"
            echo
            exit 1
         elif [ $START_CHROMOSOME -gt $STOP_CHROMOSOME ]
         then
            usage
            echo
            echo "Error: Start chromosome $START_CHROMOSOME is greater than stop chromosome $STOP_CHROMOSOME!"
            echo
            exit 1
         fi
         shift; shift
         ;;
      --phen)
         if [ -f $2 ]
         then
            PHENOTYPE_FILE=$2
         else
            info
            echo
            echo "Error: Phenotype file $2 not found!"
            echo
            exit 1
         fi
         shift
         ;;
      --analysis)
         if [ $2 == "single" ]
         then
            SOLAR_ANALYSIS=1
         elif [ $2 == "multi" ]
         then
            SOLAR_ANALYSIS=2
         else
            usage
            echo
            echo "Error: argument after --analysis must be single or multi!"
            echo
            exit 1
         fi
         shift
         ;;
      --ibd)
         CALC_SOLAR_IBDS=1
         ;;
      --ibd-dir)
         IBD_DIR=$2
         shift
         ;;
      --mibd-dir)
         MIBD_DIR=$2
         shift
         ;;
      --ibd-only)
         CALC_SOLAR_IBDS=1
         CALC_SOLAR_IBDS_ONLY=1
         ;;
      --polygenic-only)
         POLYGENIC_ONLY=1
         ;;
      --interval)
         INTERVAL=$2
         shift
         ;;
      --finemap)
         FINEMAP=$2
         shift
         ;;
      --pedcheck-cmd)
         PEDCHECK_COMMAND=$2
         shift
         ;;
      --skip-pedcheck)
         PEDCHECK_SKIP=1
         ;;
      --pedcheck-level)
         PEDCHECK_LEVEL=$2
         shift
         ;;
      --debug)
         DEBUG=1
         ;;
      --verbose)
         VERBOSE=1
         ;;
      --help|-h)
         usage
         exit 1
         ;;
      *)
         usage
         echo "Error: Unknown option:" $1
         exit 1
         ;;
   esac
   shift
done

info
analysis_options

#
# verbose debug
#
if [ $DEBUG -eq 1 ]
then
   debug
fi

#
# Create results folder
#
if [ ! -d $RESULTS_FOLDER ]
then
   mkdir $RESULTS_FOLDER
fi

#
# Chromosomes...
#
for ((CHROMOSOME=$START_CHROMOSOME ; CHROMOSOME <=$STOP_CHROMOSOME  ; CHROMOSOME++))
do

   echo "################################################################################"
   echo "# CHROMOSOME $CHROMOSOME"
   echo "################################################################################"

   #
   # Input files
   #
   PEDIGREEFILE="chr"$CHROMOSOME"."$PEDIGREE_FILE_EXTENSION
   MAPFILE="map"$CHROMOSOME".dat"

   #
   # CHROMOSOME-X
   #
   if [ $CHROMOSOME -eq 23 ]
   then
      DOWNCODE_OPTIONS=$DOWNCODE_OPTIONS" --x "
   fi

   #
   # If pedigree file exists, start analysis
   #
   if [ -f $PEDIGREEFILE ] && [ -f $MAPFILE ] && [ -f $PHENOTYPE_FILE ] && [ -f $SOLAR_USER_COMMAND_FILE ]
   then

      #
      # Count number of markers from pedigree file
      #
      if [ $DICHOTOMOUS_TRAIT -eq 1 ]
      then
         NUMBER_OF_LOCI=`awk 'END {print (NF - 6)/2}' $PEDIGREEFILE`
      else
         NUMBER_OF_LOCI=`awk 'END {print (NF - 5)/2}' $PEDIGREEFILE`
      fi

      #
      # Chromosome directory
      #
	   CHROMOSOMEDIR="chr"$CHROMOSOME
      if [ ! -d $CHROMOSOMEDIR ]
      then
         mkdir $CHROMOSOMEDIR
      fi

	   #
      # Output files
      #
      if [ $MALES -eq 1 ]
      then
         SOLAR_SINGLEPOINT_OUT="solar_singlepoint_"$CHROMOSOME"_males.out"
         SOLAR_MULTIPOINT_OUT="solar_multipoint_"$CHROMOSOME"_males.out"
      elif [ $FEMALES -eq 1 ]
	   then
         SOLAR_SINGLEPOINT_OUT="solar_singlepoint_"$CHROMOSOME"_females.out"
         SOLAR_MULTIPOINT_OUT="solar_multipoint_"$CHROMOSOME"_females.out"
      else
         SOLAR_SINGLEPOINT_OUT="solar_singlepoint_"$CHROMOSOME".out"
         SOLAR_MULTIPOINT_OUT="solar_multipoint_"$CHROMOSOME".out"
      fi

      echo "--------------------------------------------------------------------------------"
      echo " NUMBER OF LOCI $NUMBER_OF_LOCI "
      echo "--------------------------------------------------------------------------------"

	   if [ -d $CHROMOSOMEDIR ]
	   then
         #
         # Change to chromosome directory
         #
	      cd $CHROMOSOMEDIR

         #
         # PedConvert
         #
         echo "--------------------------------------------------------------------------------"
         echo " PEDCONVERT "
         echo "--------------------------------------------------------------------------------"
         $PEDCONVERT_COMMAND -p ../$PEDIGREEFILE -f ../$PHENOTYPE_FILE  -m ../$MAPFILE -n $NUMBER_OF_LOCI $PEDCONVERT_OPTIONS

         #
         # Mendelian check
         # 
         if [ $PEDCHECK_SKIP -eq 0 ]
         then
            mendelian_check
         fi
         if [ $MENDELIAN_ERRORS_FOUND -eq 0 ]
         then        
            #
            # DownCode
            #
            echo "--------------------------------------------------------------------------------"
            echo " DOWNCODE "
            echo "--------------------------------------------------------------------------------"
            $DOWNCODE_COMMAND -p ../$PEDIGREEFILE -map ../$MAPFILE -n $NUMBER_OF_LOCI --pre $DOWNCODE_OPTIONS

            #
            # Solar input files
            #
            SOLAR_PEDIGREEFILE="solar.ped"
            SOLAR_PHENOTYPEFILE="solar.phen"
            SOLAR_MAKERFILE="solar.marker"
            SOLAR_MAPFILE="solar.map"
            SOLAR_FREQFILE="solar.freq"

            #
            # Solar command file
            #
            if [ $SOLAR_ALL_USER_COMMAND_USED -eq 0 ]
            then
               echo "--------------------------------------------------------------------------------"
               echo " Creating solar command file "
               echo "--------------------------------------------------------------------------------"
               echo "outdir $SOLAR_OUTDIR" > $SOLAR_COMMAND_FILE
               echo "load pedigree "$SOLAR_PEDIGREEFILE >> $SOLAR_COMMAND_FILE
               if [ $CHROMOSOME -eq 23 ]
               then
                  echo "load markers -xlinked "$SOLAR_MAKERFILE >> $SOLAR_COMMAND_FILE
               else
                  echo "load markers "$SOLAR_MAKERFILE >> $SOLAR_COMMAND_FILE
               fi
               echo "load map -haldane "$SOLAR_MAPFILE >> $SOLAR_COMMAND_FILE
               echo "load freq "$SOLAR_FREQFILE >> $SOLAR_COMMAND_FILE
               echo "load phenotypes "$SOLAR_PHENOTYPEFILE >> $SOLAR_COMMAND_FILE
               
               #
               # User commands!
               #
               cat ../$SOLAR_USER_COMMAND_FILE >> $SOLAR_COMMAND_FILE
               
               if [ $POLYGENIC_ONLY -eq 0 ]
               then
                  if [ $CALC_SOLAR_IBDS -eq 1 ]
                  then
                     #IBD_DIR=`pwd`"/ibd"
                     #MIBD_DIR=`pwd`"/mibd"
                     echo "exec mkdir $IBD_DIR" >> $SOLAR_COMMAND_FILE
                     echo "ibddir $IBD_DIR" >> $SOLAR_COMMAND_FILE
                     echo "exec mkdir $MIBD_DIR" >> $SOLAR_COMMAND_FILE
                     echo "mibddir $MIBD_DIR" >> $SOLAR_COMMAND_FILE
                     echo "verbosity min" >> $SOLAR_COMMAND_FILE            
                     echo "ibd -nomle" >> $SOLAR_COMMAND_FILE
                     echo "mibd 1" >> $SOLAR_COMMAND_FILE
                     echo "verbosity default" >> $SOLAR_COMMAND_FILE            
                  else
                     if [ -d $IBD_DIR ]
                     then
                        echo "ibddir $IBD_DIR" >> $SOLAR_COMMAND_FILE
                     else
                        echo "IBD dir not found ($IBD_DIR) ! Make sure it's right path or you have to calculate IBDs!"
                        break;
                     fi
                     if [ -d $MIBD_DIR ]
                     then
                        echo "mibddir $MIBD_DIR" >> $SOLAR_COMMAND_FILE
                     else
                        echo "MIBD dir not found ($MIBD_DIR) ! Make sure it's right path or you have to calculate MIBDs!"
                        break;
                     fi
                  fi
                  if [ $CALC_SOLAR_IBDS_ONLY -eq 0 ]
                  then
                     if [ $SOLAR_ANALYSIS -eq 0 ] || [ $SOLAR_ANALYSIS -eq 1 ] 
                     then
                        echo "twopoint" >> $SOLAR_COMMAND_FILE
                     fi
                     if [ $SOLAR_ANALYSIS -eq 0 ] || [ $SOLAR_ANALYSIS -eq 2 ]
                     then
                        echo "interval $INTERVAL" >> $SOLAR_COMMAND_FILE
                        if [ $FINEMAP -ne -1 ] 
                        then
                           echo "interval $FINEMAP" >> $SOLAR_COMMAND_FILE                        
                        fi
                        echo "chromosome $CHROMOSOME" >> $SOLAR_COMMAND_FILE
                        echo "multipoint -ov" >> $SOLAR_COMMAND_FILE
                     fi
                  fi
               fi
            else
               cat ../$SOLAR_USER_COMMAND_FILE > $SOLAR_COMMAND_FILE
            fi
            #
            # Solar analysis
            #
            if [ $MAKE_FILES_ONLY -eq 0 ]
            then
               echo "################################################################################"
               echo "# Solar (ANALYSIS)"
               echo "################################################################################"
               if [ -f $SOLAR_PEDIGREEFILE ] && [ -f $SOLAR_MARKERFILE ] && [ -f $SOLAR_MAPFILE ] && [ -f $SOLAR_FREQFILE ] && [ -f $SOLAR_PHENOTYPEFILE ]
               then
                  echo "################################################################################"
                  echo "# Solar (ANALYSIS)"
                  echo "################################################################################"             
                  echo "Commands used:"
                  echo "=============="
                  cat $SOLAR_COMMAND_FILE
                  echo "Analyzing..."
                  $SOLAR_COMMAND < $SOLAR_COMMAND_FILE
                  echo "...done"
                  if [ $POLYGENIC_ONLY -eq 1 ]
                  then
                     echo
                     echo "Polygenic screening done, exiting..."
                     echo
                     exit
                  fi
               else
                  echo "Warning: Input files are missing: $SOLAR_PEDIGREEFILE, $SOLAR_MARKERFILE, $SOLAR_MAPFILE, $SOLAR_FREQFILE, $SOLAR_PHENOTYPEFILE"
               fi
               #
               # Move results
               #
               echo
               echo "Moving results files...if any.."
               echo
               if [ -f $SOLAR_OUTDIR/twopoint.out ]
               then
                  mv -v $SOLAR_OUTDIR/twopoint.out ../$RESULTS_FOLDER/$SOLAR_SINGLEPOINT_OUT           
               fi
               if [ -f $SOLAR_OUTDIR/multipoint1.out ]
               then
               mv -v $SOLAR_OUTDIR/multipoint1.out ../$RESULTS_FOLDER/$SOLAR_MULTIPOINT_OUT
               fi
            fi
         else
            echo "--------------------------------------------------------------------------------"
            echo " Mendelian error(s) found, see pedcheck.err, skipping this chromosome..."
            echo "--------------------------------------------------------------------------------"         
         fi
         cd ..        
      fi
   else
      echo "Skipping...no pedigree file $PEDIGREEFILE or map file $MAPFILE or phenotype file $PHENOTYPE_FILE or SOLAR command file $SOLAR_USER_COMMAND_FILE found!"
   fi
done

echo
echo "End of the AutoSolar!"
echo
