/*
  Title     : downcode.cpp
  Version   : 1.0
  Copyright : 2004 Tero Hiekkalinna
  Author    : Tero Hiekkalinna (tero.hiekkalinna@helsinki.fi)
  Date      : 25.1.2005
*/

#include <fstream>
#include <algorithm>

#include "DownCode.h"
#include "Utils.h"

#define MAX_MARKERS 800

// welcome message
void welcome()
{
   cout << "-------------------------------------------------------------------------------" << endl;
   cout << " Program DownCode 1.0 - 25th of January 2005" << endl;
   cout << " Copyright (C) 2004-2005 Tero Hiekkalinna, Tero.Hiekkalinna@Helsinki.FI" << endl;
   cout << " Program for estimating (and downcoding) allele frequencies from pedigree file" << endl;
   cout << "-------------------------------------------------------------------------------" << endl;
   cout << endl;
}

int main( int argc, char * argv[] )
{
   welcome();
   DownCode *DC = new DownCode();
   if(DC==NULL)
   {
      cout << "Error: Unable to allocate memory...aborting" << endl;
      return -1;
   }
   bool status;
   status = DC->ParseCommandLineArguments(argc,argv);
   if(status==false)
   {
      return -1;
   }
   DC->initializeStuff();
   status = DC->ReadPedigreeFile();
   if(status==false)
   {
      return -1;
   }
   status = DC->ReadMapFile();
   if(status==false)
   {
      return -1;
   }

   DC->CountAlleles();

   DC->EstimateAlleleFrequencies();
   DC->CompareAlleleFrequencies();

   if(DC->getProgram() == "solar")
   {
      DC->WriteSolarFrequencyFile();
   }
   else if(DC->getProgram() == "analyze" || DC->getProgram()=="genehunter")
   {
      DC->CreateDowncodeTable();
      DC->WriteFrequencyandDownCodeTable();
      DC->WriteLinkageLocusFile();
      DC->WritePedigreeFile();
   }
   else
   {
      cout << "Warning: Output format/program not specified!" << endl;
   }
   return 0;
}

DownCode::DownCode()
{
   this->PedigreeFile = "inped.dat";
   this->LocusFile = "indata.dat";
   this->ModelFile = "model.dat";
   this->MapFile = "map.dat";

   this->OutputPedigreeFile = "pedin.dat";
   this->OutputLocusFile = "datain.dat";
   this->OutputSolarFrequencyFile = "solar.freq";
   this->OutputDownCodeFrequencyFile = "downcode.out";

   this->PedFileFormat = "post";
   this->Program = "analyze";
   this->Frequency = "all";
   this->trait = false;
   this->xlinked = false;
   this->liab = false;

   this->number_of_markers = -1;

   initializeMissingAlleleVector();
}


bool DownCode::ParseCommandLineArguments(int argc, char * argv[])
{
   /*
    * Let's parse argument list...
    */
   string argument;
   for(int index = 1 ; index < argc ; index++)
   {
      argument = argv[index];

      // Pedigree file
      if(argument == "-p" && argc >= index+1)
      {
         this->PedigreeFile = argv[index+1];
         index++;
         cout << "Pedigree file is " << this->PedigreeFile << endl;
      }

      // Locus file
      else if(argument == "-d" && argc >= index+1)
      {
         this->LocusFile = argv[index+1];
         index++;
         cout << "Locus file is " << this->LocusFile << endl;
      }

      // Model file
      else if(argument == "-m" && argc >= index+1)
      {
         this->ModelFile = argv[index+1];
         index++;
         cout << "Model file is " << this->ModelFile << endl;
      }

      // Map file
      else if(argument == "-map" && argc >= index+1)
      {
         this->MapFile = argv[index+1];
         index++;
         cout << "Map file is " << this->MapFile << endl;
      }

      // Number of marker in pedigree file
      else if(argument == "-n" && argc >= index+1)
      {
         int markers = atoi(argv[index+1]);
         if(markers > 0 && markers <= MAX_MARKERS)
         {
            this->number_of_markers = markers;
            index++;
         }
         else
         {
            cout << "Error: Illegal number of markers (MAX is " << MAX_MARKERS << "), or " << argv[index+1] << " not a number!" << endl;
            return false;
         }
         cout << "Number of markers is " << this->number_of_markers << endl;
      }

      // pre-Makeped file format
      else if(argument == "--pre")
      {
         this->PedFileFormat = "pre";
         cout << "Pedigree file format is pre-MAKEPED" << endl;
      }

      // post-Makeped file format
      else if(argument == "--post")
      {
         this->PedFileFormat = "post";
         cout << "Pedigree file format is post-MAKEPED" << endl;
      }

      // Ignore phenotype from pedigree file
      else if(argument == "--trait")
      {
         this->trait = true;
         cout << "Affection status phenotype in pedigree file" << endl;
      }

      // X-Linked?
      else if(argument == "--x")
      {
         this->xlinked = true;
         cout << "X-chromosomal data" << endl;
      }

      // Liability class
      else if(argument == "--liab")
      {
         this->liab = true;
         cout << "Pedigree file has liability class" << endl;
      }

      // All indivudual allele frequencies
      else if(argument == "--all")
      {
         this->Frequency = "all";
         cout << "All indivudual allele frequencies are used" << endl;
      }

      // Founder allele frequencies
      else if(argument == "--founders")
      {
         this->Frequency = "founders";
         cout << "Founder allele frequencies are used" << endl;
      }

      // Output format option
      else if(argument == "--prog" && argc >= index+1)
      {
         string program = argv[index+1];
         if(program == "solar" || program == "genehunter" || program == "analyze")
         {
            this->Program = program;
            cout << "Output file format is: " << this->Program << endl;
         }
         else
         {
            cout << "Error: Illegal program name... : " << program << endl;
            return false;
         }
         index++;
         cout << "Output allele frequency file format is " << this->Program << endl;
      }

      // If argument is illegal...
      else
      {
         cout << "Error: Illegal command line argument \"" << argument << "\"" << endl;
         cout << endl;
         return false;
      }
   }

   // Everything OK!
   return true;
}

void DownCode::initializeStuff()
{
   if(this->Program == "genehunter")
   {
      this->OutputPedigreeFile = "gh_coded.ped";
      this->OutputLocusFile = "gh.dat";
   }
   else if(this->Program == "analyze")
   {
      if(this->PedFileFormat=="post")
     {
         this->OutputPedigreeFile = "pedin.dat";
         this->OutputLocusFile = "datain.dat";
      }
      else if(this->PedFileFormat=="pre")
      {
         this->OutputPedigreeFile = "analyze_coded.ped";
         this->OutputLocusFile = "datain.dat";
      }
   }
   else if(this->Program == "solar")
   {
      this->OutputSolarFrequencyFile = "solar.freq";
   }
   else
   {
      cout << "Error: Illegal program name... : " << this->Program << endl;
      return;
   }
}

bool DownCode::ReadPedigreeFile()
{
   /*
    * Open pedigree file for reading
    */
   cout << "Reading pedigree file...";

   ifstream PedigreeIn(this->PedigreeFile);
   if(!PedigreeIn)
   {
      cout << "Error: Unable to open pedigree file " << this->PedigreeFile << " for reading." << endl;
      return false;
   }

   /*
    * Let's read pedigree file
    */

   // Temporary variables
   Person   *person;
   string familyid, personid, fatherid, motherid;
   string fos, nps, nms;
   string sex;
   string proband;
   string affection_status, liability_class;
   string allele[2];

   while(!PedigreeIn.eof())
   {
      // read person information
      PedigreeIn >> familyid >> personid >> fatherid >> motherid;

      // post-MAKEPED format
      if(this->PedFileFormat=="post")
      {
         PedigreeIn >> fos >> nps >> nms >> sex >> proband;
      }
      else // pre-MAKEPED
      {
         PedigreeIn >> sex;
      }

      // read dicotomous trait column
      if(this->trait == true)
      {
         PedigreeIn >> affection_status;
      }

      // read liability class column
      if(this->liab == true)
      {
         PedigreeIn >> liability_class;
      }

      /*
       * This is needed or last record is inserted 2 times
       */
      if(PedigreeIn.eof())
         break;

      /*
       * Allocate person and insert.
       */
      person = new Person(familyid,personid,fatherid,motherid,sex);

      // check allocation
      if(person==NULL)
      {
         cout << "Error: Cannot allocate memory for person, aborting..." << endl;
         return false;
      }

      // Add other info
      if(this->PedFileFormat=="post")
      {
         person->setFirstOffspring(fos);
         person->setNextPaternalSibling(nps);
         person->setNextMaternalSibling(nms);
         person->setProbandStatus(proband);
         person->setPedFileFormat("post");
      }
      if(this->trait == true)
      {
         person->setAffectionStatus(affection_status);
      }
      if(this->liab == true)
      {
         person->setLiabilityClass(liability_class);
      }

      individuals.push_back(person);

      /*
       * Next we read genotypes (number_of_markers * 2, since each genotype
       * has two alleles)
       */
      for(int marker = 0 ; marker < this->number_of_markers ; marker++)
      {
         PedigreeIn >> allele[0] >> allele[1];
         person->insertGenotype(allele[0],allele[1]);
      }
   }
   PedigreeIn.close();

   cout << this->individuals.size() << " individuals read." << endl;

   //PrintPedigrees();
   return true;
}

bool DownCode::ReadMapFile()
{
   /*
    * Map file
    */
   Map marker_pos;
   string chr, mname;
   double position;
   bool isMarker;
   char line[10001];

   cout << "Reading map file...";
   ifstream mapin(this->MapFile);
   if(!mapin)
   {
      cout << "Error: Unable to open map file " << this->MapFile << " for reading." << endl;
      return false;
   }
   mapin.getline(line,10000); // Header
   while(!mapin.eof())
   {
      mapin >> chr >> position >> mname;

      /*
       * This is needed or last record is inserted 2 times
       */
      if(mapin.eof())
         break;

      isMarker = false;
      for(unsigned int i = 0 ; i < map.size() ; i++)
      {
         if(mname == map[i].getName())
         {
            isMarker = true;
         }
      }

      if(isMarker==true)
      {
         cout << "Error: Marker \"" << mname << "\" already read (ignoring marker), double entry in map file?" << endl;
         return false;
      }
      else
      {
         marker_pos = Map(chr,position,mname);
         map.push_back(marker_pos);
      }

   }
   mapin.close();
   cout << map.size() << " markers read, done." << endl;
   return true;
}

void DownCode::CountAlleles()
{
   vector<Allele> allele_count_all;
   vector<Allele> allele_count_founders;
   string m_allele[2];
   bool found;

   for(int marker = 0 ; marker < this->number_of_markers ; marker++)
   {
      for(unsigned int i = 0 ; i < this->individuals.size() ; i++)
      {
         for(int j = 0 ; j < 2 ; j++)
         {
            if(this->xlinked==true && individuals[i]->getSex() == "1" && j == 1) // If Xlinked, only one allele is counted from males!
            {
               //cout << "X-linked ::: Person: " << individuals[i]->getPersonID() << " sex: " << individuals[i]->getSex() << " j: " << j << endl;
            }
            else
            {
               m_allele[j] = this->individuals[i]->getMarkerAllele(marker,j);

               // All individual allele counts
               found=false;
               for(unsigned int allele = 0 ; allele < allele_count_all.size() ; allele++)
               {
                  if(allele_count_all[allele].getName() == m_allele[j] && isMissingAllele(m_allele[j])==false)
                  {
                     allele_count_all[allele].addCount();
                     found=true;
                     break;
                  }
               }

               if(found==false && isMissingAllele(m_allele[j])==false)
               {
                  Allele A = Allele(m_allele[j]);
                  A.addCount();
                  allele_count_all.push_back(A);

                  Allele B = Allele(m_allele[j]);
                  allele_count_founders.push_back(B);
               }

               // Founder allele counts
               if(this->individuals[i]->getFatherID() == "0" && this->individuals[i]->getMotherID() == "0")
               {
                  for(unsigned int allele = 0 ; allele < allele_count_founders.size() ; allele++)
                  {
                     if(allele_count_all[allele].getName() == m_allele[j] && isMissingAllele(m_allele[j])==false)
                     {
                        allele_count_founders[allele].addCount();
                        break;
                     }
                  }
               }
            }
         }
      }
      // Sort
      sortAlleleVector(allele_count_all);
      sortAlleleVector(allele_count_founders);

      allele_freqs_all.push_back(allele_count_all);
      allele_freqs_founders.push_back(allele_count_founders);
      allele_count_all.clear();
      allele_count_founders.clear();
   }
   //PrintAlleleCounts();
}

void DownCode::sortAlleleVector(vector<Allele> &alleles)
{
   string allele[2];
   bool first_allele_number;
   bool second_allele_number;
   int allele_int[2];
   bool swap_em;

   if(alleles.size()==0)
      return;

   for(unsigned int i = 0 ; i < alleles.size() - 1 ; i++)
   {
      for(unsigned int j = 0 ; j < alleles.size() - 1 - i; j++)
      {
         first_allele_number  = true;
         second_allele_number = true;

         /*
          * Is first allele a integer number
          */
         allele[0] = alleles[j+1].getName();
         const char * allele_1 = allele[0].c_str();
         for(unsigned int k = 0 ; k < allele[0].length() ; k++)
         {
            if(isdigit(allele_1[k])==0)
            {
               first_allele_number = false;
               break;
            }
         }

         /*
          * Is second allele a integer number
          */
         allele[1] = alleles[j].getName();
         const char * allele_2 = allele[1].c_str();
         for(unsigned int k = 0 ; k < allele[1].length() ; k++)
         {
            if(isdigit(allele_2[k])==0)
            {
               second_allele_number = false;
               break;
            }
         }

         /*
          * Is both are integers, convert them to ints and let's do integer comparison
          */
         if(first_allele_number == true && second_allele_number == true)
         {
            allele_int[0] = atoi(allele_1);
            allele_int[1] = atoi(allele_2);

            if(allele_int[0] < allele_int[1])
            {
               swap_em = true;
            }
            else
            {
               swap_em = false;
            }
         }
         else
         {
            if(allele[0].compare(allele[1]) < 0)
            {
               swap_em = true;
            }
            else
            {
               swap_em = false;
            }
         }

         if(swap_em == true)
         {
            Allele tmp = alleles[j];
            alleles[j] = alleles[j+1];
            alleles[j+1] = tmp;
         }
      }
   }
}

void DownCode::initializeMissingAlleleVector()
{
   this->missing_allele.push_back("0");
   this->missing_allele.push_back("x");
   this->missing_allele.push_back("X");
   this->missing_allele.push_back(".");
   this->missing_allele.push_back("-");
}

bool DownCode::isMissingAllele(string allele)
{
   for(unsigned int i = 0 ; i < this->missing_allele.size() ; i++)
   {
      if(this->missing_allele[i] == allele)
      {
         return true;
      }
   }
   return false;
}

void DownCode::EstimateAlleleFrequencies()
{
   int total;
   double freq;
   double freq_total;

   for(unsigned int marker = 0 ; marker < allele_freqs_all.size() ; marker++)
   {
      total = 0;
      for(unsigned int allele = 0 ; allele < allele_freqs_all[marker].size() ; allele++)
      {
         if(allele_freqs_all[marker][allele].getName() != "0")
         {
            total += allele_freqs_all[marker][allele].getCount();
         }
      }

      freq_total = 0.0;
      for(unsigned int allele = 0 ; allele < allele_freqs_all[marker].size() ; allele++)
      {
         if(allele_freqs_all[marker][allele].getName() != "0")
         {
            freq = allele_freqs_all[marker][allele].getCount() / (1.0 * total);
            if(allele == allele_freqs_all[marker].size()-1)
            {
               allele_freqs_all[marker][allele].setFrequency(1.0-freq_total);
            }
            else
            {
               allele_freqs_all[marker][allele].setFrequency(freq);
               freq_total += freq;
            }
         }
      }
   }

   for(unsigned int marker = 0 ; marker < allele_freqs_founders.size() ; marker++)
   {
      total = 0;
      for(unsigned int allele = 0 ; allele < allele_freqs_founders[marker].size() ; allele++)
      {
         if(allele_freqs_founders[marker][allele].getName() != "0")
         {
            total += allele_freqs_founders[marker][allele].getCount();
         }
      }

      for(unsigned int allele = 0 ; allele < allele_freqs_founders[marker].size() ; allele++)
      {
         if(allele_freqs_founders[marker][allele].getName() != "0")
         {
            if(total!=0)
            {
               freq = allele_freqs_founders[marker][allele].getCount() / (1.0 * total);
            }
            else
            {
               freq=0.0;
            }
            allele_freqs_founders[marker][allele].setFrequency(freq);
         }
      }
   }
   //PrintAlleleFrequencies();
}

bool DownCode::CompareAlleleFrequencies()
{
   for(unsigned int marker = 0 ; marker < allele_freqs_all.size() ; marker++)
   {
      for(unsigned int allele = 0 ; allele < allele_freqs_all[marker].size() ; allele++)
      {
         if(allele_freqs_founders[marker][allele].getFrequency() == 0.0)
         {
            cout << "Warning: Founder allele frequencies cannot be used, founders do not have all alleles! Using all individual allele frequencies." << endl;
            this->Frequency="all";
            return false;
         }
      }
   }
   return true;
}

void DownCode::CreateDowncodeTable()
{
   vector<DownCodeTable> dcode;
   DownCodeTable dctable;
   string old_allele;
   string new_allele;
   int new_allele_number;

   // all
   for(unsigned int marker = 0 ; marker < allele_freqs_all.size() ; marker++)
   {
      new_allele_number=1;
      for(unsigned int allele = 0 ; allele < allele_freqs_all[marker].size() ; allele++)
      {
         old_allele = allele_freqs_all[marker][allele].getName();
         if(old_allele != "0")
         {
            new_allele = IntToString(new_allele_number);
            dctable = DownCodeTable(old_allele,new_allele);
            new_allele_number++;
            dcode.push_back(dctable);
         }
      }
      downcode_table_all.push_back(dcode);
      dcode.clear();
   }

   // founders
   for(unsigned int marker = 0 ; marker < allele_freqs_founders.size() ; marker++)
   {
      new_allele_number=1;
      for(unsigned int allele = 0 ; allele < allele_freqs_founders[marker].size() ; allele++)
      {
         old_allele = allele_freqs_founders[marker][allele].getName();
         if(old_allele != "0")
         {
            new_allele = IntToString(new_allele_number);
            dctable = DownCodeTable(old_allele,new_allele);
            new_allele_number++;
            dcode.push_back(dctable);
         }
      }
      downcode_table_founders.push_back(dcode);
      dcode.clear();
   }
   //PrintDowncodeTable();
}

void DownCode::SortDowncodeTable()
{
   cout << "Sorting DownCodeTable " << endl; // TODO
   for(unsigned int marker = 0 ; marker < allele_freqs_all.size() ; marker++)
   {
      cout << "Marker " << marker + 1 << endl;
      for(unsigned int allele = 0 ; allele < allele_freqs_all[marker].size() ; allele++)
      {
         if(allele_freqs_all[marker][allele].getName() == "0")
         {
            //vector<Allele>::iterator p1 = find(allele_freqs_all[marker].begin(),allele_freqs_all[marker].end(),getName()=="0");
            //allele_freqs_all[marker].erase(find(allele_freqs_all[marker].begin(),allele_freqs_all[marker].end(),initial('0')));
            //allele_freqs_all[marker].erase(allele_freqs_all[marker].begin());
            allele_freqs_all[marker].erase(allele_freqs_all[marker].end()+1);
         }

      }
   }
}

bool DownCode::WritePedigreeFile()
{
   cout << "Writing LINKAGE pedigree file...(" << this->OutputPedigreeFile << ")...";
   fstream LinkagePedigreeFile(this->OutputPedigreeFile,ios_base::out);
   if(LinkagePedigreeFile==NULL)
   {
      cout << "Error: Unable to open " << this->OutputPedigreeFile << " for writing!" << endl;
      return false;
   }
   string old_allele;
   string new_allele;
   for(unsigned int i = 0 ; i < this->individuals.size() ; i++)
   {
      LinkagePedigreeFile << this->individuals[i]->getInString();
      for(int marker = 0 ; marker < this->number_of_markers ; marker++)
      {
         for(int allele = 0 ; allele < 2 ; allele++)
         {
            old_allele = this->individuals[i]->getMarkerAllele(marker,allele);
            new_allele = getNewMarkerAlleleCode(marker,old_allele);
            LinkagePedigreeFile << " " << new_allele;
         }
      }
      LinkagePedigreeFile << endl;
   }
   LinkagePedigreeFile.close();
   cout << "done" << endl;
   return true;
}

string DownCode::getNewMarkerAlleleCode(int marker, string old_allele)
{
   if(marker > downcode_table_all.size())
   {
      cout << "Error: Marker index out of bounds!" << endl;
      return "error";
   }
   for(unsigned int allele = 0 ; allele < downcode_table_all[marker].size() ; allele++)
   {
      if(downcode_table_all[marker][allele].getOld() == old_allele)
     {
         return downcode_table_all[marker][allele].getNew();
     }
   }
   return "0"; // MISSING ALLELE, hah! :)
}

bool DownCode::WriteFrequenciesLinkageLocusFile()
{
   cout << "Writing frequencies out... LINKAGE format...";
   fstream LinkageLocusFile(this->OutputLocusFile,ios_base::out);
   if(LinkageLocusFile==NULL)
   {
      cout << "Error: Unable to open " << this->OutputLocusFile << " for writing!" << endl;
      return false;
   }

   // This will print full float field to position column
   LinkageLocusFile.setf(ios_base::fixed,ios_base::floatfield);

   for(unsigned int marker = 0 ; marker < allele_freqs_all.size() ; marker++)
   {
      LinkageLocusFile << "3 " << allele_freqs_all[marker].size() << " # " << map[marker].getName() << endl;
      for(unsigned int allele = 0 ; allele < allele_freqs_all[marker].size() ; allele++)
      {
         if(this->Frequency=="all")
         {
            LinkageLocusFile << allele_freqs_all[marker][allele].getFrequency() << " ";
         }
         else
         {
            LinkageLocusFile << allele_freqs_founders[marker][allele].getFrequency() << " ";
         }
      }
      LinkageLocusFile << endl;
   }
   cout << "done" << endl;
   return true;
}

bool DownCode::WriteLinkageLocusFile()
{
   int type;
   double disease_allele_freq;
   int nliab;

   cout << "Reading MODEL file...(" << this->ModelFile << ")...";
   ifstream modelin(this->ModelFile);
   if(!modelin)
   {
      cout << endl;
      cout << "Error: Unable to open model file " << this->ModelFile << " for reading." << endl;
      return false;
   }

   modelin >> type >> disease_allele_freq >> nliab;

   if(type!=0 && type!=1)
   {
      cout << endl;
      cout << "Error in model file: Wrong type in Autosomal/X-Linked identifier: " << type << endl;
      return false;
   }

   if(disease_allele_freq < 0.0 || disease_allele_freq > 1.0)
   {
      cout << endl;
      cout << "Error in model file: Disease allele frequency out of range: " << disease_allele_freq << endl;
      return false;
   }

   if(nliab < 1)
   {
      cout << endl;
      cout << "Error in model file: Number of liability classes < 1: " << nliab << endl;
      return false;
   }

   cout << "writing LINKAGE locus file...(" << this->OutputLocusFile << ")...";
   fstream LocusFile(this->OutputLocusFile,ios_base::out);
   if(LocusFile==NULL)
   {
      cout << endl;
      cout << "Error: Unable to open " << this->OutputLocusFile << " for writing!" << endl;
      return -1;
   }

   // This will print full float field to position column
   LocusFile.setf(ios_base::fixed,ios_base::floatfield);

   LocusFile << this->number_of_markers + 1 << " 0 " << type << " 5 << NO. OF LOCI, RISK LOCUS, SEXLINKED (IF 1) PROGRAM" << endl;
   LocusFile << "0 0.0 0.0 0 << MUT LOCUS, MUT MALE, MUT FEM, HAP FREQ (IF 1)";
   LocusFile << endl;
   for(int i = 1 ; i <= this->number_of_markers + 1 ; i++)
   {
        LocusFile << " " << i;
   }
   LocusFile << endl;
   LocusFile << "1 2 << AFFECTION, NO. OF ALLELES" << endl;
   LocusFile << 1.0-disease_allele_freq << " " << disease_allele_freq << " << GENE FREQUENCIES" << endl;
   LocusFile << nliab << endl;
   double pen[3];
   for(int i = 0 ; i < nliab ; i++)
   {
      modelin >> pen[0] >> pen[1] >> pen[2];
      for(int j = 0 ; j < 3 ; j++)
      {
         if(pen[j] < 0.0 || pen[j] > 1.0)
         {
            cout << "Error in model file: penetrance out of range: " << pen[j] << endl;
            return false;
         }
      }
      LocusFile << pen[0] << " " << pen[1] << " " << pen[2] << " << PENETRANCES" << endl;

      // X-Linked
      if(type==1)
      {
         modelin >> pen[0] >> pen[1];
         for(int j = 0 ; j < 2 ; j++)
         {
            if(pen[j] < 0.0 || pen[j] > 1.0)
            {
               cout << "Error in model file: penetrance out of range: " << pen[j] << endl;
               return false;
            }
         }
         LocusFile << pen[0] << " " << pen[1] << " << PENETRANCES" << endl;
      }
   }

   modelin.close();


   for(unsigned int marker = 0 ; marker < allele_freqs_all.size() ; marker++)
   {
      LocusFile << "3 " << allele_freqs_all[marker].size() << " # " << map[marker].getName() << endl;
      for(unsigned int allele = 0 ; allele < allele_freqs_all[marker].size() ; allele++)
      {
         if(this->Frequency=="all")
         {
            LocusFile << allele_freqs_all[marker][allele].getFrequency() << " ";
         }
         else
         {
            LocusFile << allele_freqs_founders[marker][allele].getFrequency() << " ";
         }
      }
      LocusFile << endl;
   }
   LocusFile << "0 0" << endl;
   LocusFile << "0.5";
   double recombination_fraction;
   for(unsigned int i = 0 ; i < map.size() - 1 ; i++)
   {
      recombination_fraction = HaldaneCentiMorganToRecombinationFraction(map[i+1].getPosition() - map[i].getPosition());
      if(recombination_fraction < 0)
      {
         cout << "Warning: negative recombinationfraction ... problem with map file?" << endl;
      }
      if(recombination_fraction == 0.0)
      {
         cout << "Warning: Recombinationfraction zero between markers ... problem with map file?" << endl;
      }
      LocusFile << " " << recombination_fraction;
   }
   LocusFile << endl;
   LocusFile << "1 0.1 0.45" << endl;
   LocusFile.close();

   cout << "done" << endl;

   return true;
}

bool DownCode::WriteSolarFrequencyFile()
{
   cout << "Writing SOLAR frequency file...";
   fstream SolarFreqFile(this->OutputSolarFrequencyFile,ios_base::out);
   if(SolarFreqFile==NULL)
   {
      cout << "Error: Unable to open " << this->OutputSolarFrequencyFile << " for writing!" << endl;
      return false;
   }

   // This will print full float field to position column
   SolarFreqFile.setf(ios_base::fixed,ios_base::floatfield);

   for(unsigned int marker = 0 ; marker < allele_freqs_all.size() ; marker++)
   {
      SolarFreqFile << map[marker].getName() << " ";
      for(unsigned int allele = 0 ; allele < allele_freqs_all[marker].size() ; allele++)
      {
         SolarFreqFile << allele_freqs_all[marker][allele].getName() << " ";
         if(this->Frequency=="all")
         {
            SolarFreqFile << allele_freqs_all[marker][allele].getFrequency() << " ";
         }
         else
         {
            SolarFreqFile << allele_freqs_founders[marker][allele].getFrequency() << " ";
         }
      }
      SolarFreqFile << endl;
   }
   cout << "done" << endl;
   return true;
}

string DownCode::getProgram()
{
   return this->Program;
}

void DownCode::PrintPedigrees()
{
   for(unsigned int i = 0 ; i < this->individuals.size() ; i++)
   {
        cout << this->individuals[i]->getInStringALL();
   }
}

void DownCode::PrintAlleleCounts()
{
   cout << "Allele AllCount FoundersCount" << endl;
   for(unsigned int marker = 0 ; marker < allele_freqs_all.size() ; marker++)
   {
      cout << "Marker " << marker + 1 << endl;
      for(unsigned int allele = 0 ; allele < allele_freqs_all[marker].size() ; allele++)
      {
         cout << allele_freqs_all[marker][allele].getName() << " ";
         cout << allele_freqs_all[marker][allele].getCount() << " ";
         cout << allele_freqs_founders[marker][allele].getCount() << endl;
      }
   }
}

void DownCode::PrintAlleleFrequencies()
{
   cout << "Estimated allele frequencies" << endl;
   cout << "Marker All_Count All_Freq Founders_Count Founders_Freq" << endl;
   for(unsigned int marker = 0 ; marker < allele_freqs_all.size() ; marker++)
   {
      cout << this->map[marker].getName()<< endl;
      for(unsigned int allele = 0 ; allele < allele_freqs_all[marker].size() ; allele++)
      {
         cout << allele_freqs_all[marker][allele].getName() << " ";
         cout << allele_freqs_all[marker][allele].getCount() << " ";
         cout << allele_freqs_all[marker][allele].getFrequency() << " ";
         cout << allele_freqs_founders[marker][allele].getCount() << " ";
         cout << allele_freqs_founders[marker][allele].getFrequency() << endl;
      }
   }
}

void DownCode::PrintDowncodeTable()
{
   cout << "Allele re-coding table : Old Allele -> New Allele" << endl;
   for(unsigned int marker = 0 ; marker < downcode_table_all.size() ; marker++)
   {
      cout << this->map[marker].getName()<< endl;
      for(unsigned int allele = 0 ; allele < downcode_table_all[marker].size() ; allele++)
      {
         cout << downcode_table_all[marker][allele].getOld() << " -> ";
         cout << downcode_table_all[marker][allele].getNew() << " " << endl;
      }
   }
}

bool DownCode::WriteFrequencyandDownCodeTable()
{
   cout << "Writing allele frequency / downcoding summary file...(" << this->OutputDownCodeFrequencyFile << ")" << endl;
   fstream DownCodeFile(this->OutputDownCodeFrequencyFile,ios_base::out);
   if(DownCodeFile==NULL)
   {
      cout << "Error: Unable to open " << this->OutputDownCodeFrequencyFile << " for writing!" << endl;
      return false;
   }

   DownCodeFile.setf(ios_base::fixed,ios_base::floatfield);

   DownCodeFile << "Estimated allele frequencies and downcoded allele name." << endl << endl;
   for(unsigned int marker = 0 ; marker < allele_freqs_all.size() ; marker++)
   {
      DownCodeFile << "Marker : " << map[marker].getName() << endl << endl;
      DownCodeFile.width(6);
      //DownCodeFile.fill('#');
      DownCodeFile << "Allele";
      DownCodeFile.width(11);
      //DownCodeFile.fill('#');
      DownCodeFile << "All_Count";
      DownCodeFile.width(15);
      //DownCodeFile.fill('#');
      DownCodeFile << "All_Frequency";
      DownCodeFile.width(15);
      //DownCodeFile.fill('#');
      DownCodeFile << "Founders_Count";
      DownCodeFile.width(19);
      //DownCodeFile.fill('#');
      DownCodeFile << "Founders_Frequency";
      DownCodeFile.width(12);
      //DownCodeFile.fill('#');
      DownCodeFile << "New_Allele" << endl;
      for(unsigned int allele = 0 ; allele < allele_freqs_all[marker].size() ; allele++)
      {
         DownCodeFile.width(6);
         //DownCodeFile.fill('#');
         DownCodeFile << downcode_table_all[marker][allele].getOld();
         DownCodeFile.width(11);
         //DownCodeFile.fill('#');
         DownCodeFile << allele_freqs_all[marker][allele].getCount();
         DownCodeFile.width(15);
         //DownCodeFile.fill('#');
         DownCodeFile << allele_freqs_all[marker][allele].getFrequency();
         DownCodeFile.width(15);
         //DownCodeFile.fill('#');
         DownCodeFile << allele_freqs_founders[marker][allele].getCount();
         DownCodeFile.width(19);
         //DownCodeFile.fill('#');
         DownCodeFile << allele_freqs_founders[marker][allele].getFrequency();
         DownCodeFile.width(12);
         //DownCodeFile.fill('#');
         DownCodeFile << downcode_table_all[marker][allele].getNew() << " " << endl;
      }
      DownCodeFile << endl;
   }
   return true;
}
