/*
  Title     : pedconvert.cpp
  Version   : 1.0
  Copyright : 2004 Tero Hiekkalinna
  Author    : Tero Hiekkalinna (tero.hiekkalinna@helsinki.fi)
  Date      : 4.2.2004

  58161-5 Tietorakenteiden Harjoitusty, kevt 2004
  Helsingin yliopisto, Tietojenksittelytieteen laitos
  Ohjaaja: Olaf Laczak
*/

#include <iostream>
using namespace std;
#include <fstream>
#include <limits>
#include <vector>

#include "PedConvert.h"
#include "PedigreeTree.h"
#include "PedigreeList.h"
#include "TestPedConvert.h"
#include "Map.h"

// Constants

#define MAX_PHENOTYPES 100
#define MAX_MARKERS 800  // Maximum number of markers in pedigree file
#define RBTREE "RBTREE"
#define LIST   "LIST"

/*
 Constructor
*/
PedConvert::PedConvert()
{
   this->sexCovariate = false;
   this->gender = "all";
   this->ignorePhenotype = false;
   this->allTraits = false;
   this->discrete = false;
   this->program = "NO FORMAT";
   this->ModelFileName = "model.dat";
   this->ModelFileIsSet = false;
   this->disable_map_position = -999.99;
}

/*
 Methods
*/

// set pedigree file name
void PedConvert::setPedigreeFile(char *file)
{
   this->PedigreeFileName = file;
}

// set phenotype file name
void PedConvert::setPhenotypeFile(char *file)
{
   this->PhenotypeFileName = file;
}

// set map file name
void PedConvert::setMapFile(char *file)
{
   this->MapFileName = file;
}

// set model file name
void PedConvert::setModelFile(char *file)
{
   this->ModelFileName = file;
   this->ModelFileIsSet = true;
}

// set number of marker
bool PedConvert::setNumberOfMarkers(int markers)
{
   // check number of markers
   if(markers < 0 || markers > MAX_MARKERS)
   {
      cout << "Error: Maximun number of markers exeeded: " << markers << ". Max is " << MAX_MARKERS <<  endl;
      return false;
   }
   this->numberOfMarkers = markers;
   return true;
}

// set number of phenotypes
bool PedConvert::setNumberOfPhenotypes(int phenos)
{
   // check number of phenotypes
   if(phenos < 0)
   {
      cout << "Error: Number of phenotypes cannot be negative! Value was: " << phenos << endl;
      return false;
   }
   else if(phenos > MAX_PHENOTYPES)
   {
      cout << "Error: This version only supports up to " << MAX_PHENOTYPES << endl;
      return false;
   }
   else
   {
      this->numberOfPhenotypes = phenos;
   }
   return true;
}

// save phenotype names in string array
void PedConvert::addPhenotypeName(string name)
{
   this->phenotypename.push_back(name);
}

void PedConvert::addMarkerName(string name)
{
   this->markername.push_back(name);
}

// Sex covariate
void PedConvert::setSexCovariate()
{
   this->sexCovariate = true;
}

// males of females or all
void PedConvert::setGender(string gender)
{
   this->gender = gender;
}

// dicotomous trait in pedigree file
void PedConvert::setIgnoreTrait()
{
   this->ignorePhenotype = true;
}

// Set all traits
void PedConvert::setAllTraits()
{
   this->allTraits = true;
}

// Set program / output format
void PedConvert::setProgram(string prog)
{
   this->program = prog;
}

void PedConvert::setOutputPedigreeFile(char *file)
{
   this->OutputPedigreeFileName = file;
}

void PedConvert::setOutputLocusFile(char *file)
{
   this->OutputLocusFileName = file;
}

void PedConvert::setOutputMapFile(char *file)
{
   this->OutputMapFileName = file;
}

void PedConvert::setOutputPhenotypeFile(char *file)
{
   this->OutputPhenotypeFileName = file;
}

void PedConvert::setOutputMarkerFile(char *file)
{
   this->OutputMarkerFileName = file;
}

// returns pedigree file name
char* PedConvert::getPedigreeFileName()
{
   return this->PedigreeFileName;
}

// return phenotype file name
char* PedConvert::getPhenotypeFileName()
{
   return this->PhenotypeFileName;
}

// return map file name
char* PedConvert::getMapFileName()
{
   return this->MapFileName;
}

// return model file name
char* PedConvert::getModelFileName()
{
   return this->ModelFileName;
}

bool PedConvert::isModelFileSet(){

   return this->ModelFileIsSet;
}

char* PedConvert::getOutputPhenotypeFile()
{
   return this->OutputPhenotypeFileName;
}

char* PedConvert::getOutputMarkerFile()
{
   return this->OutputMarkerFileName;
}

// return number of markers
int PedConvert::getNumberOfMarkers()
{
   return this->numberOfMarkers;
}

// return number of phenotypes
int PedConvert::getNumberOfPhenotypes()
{
   return this->numberOfPhenotypes;
}

// return number of markers in marker file
int PedConvert::getNumberOfMarkersInMapFile()
{
   return this->markername.size();
}

// return phenotype name from string array
string PedConvert::getPhenotypeName(int index)
{
   if(index < 0 || index > this->numberOfPhenotypes)
   {
      cout << "Error: Index out of bounds! Value was: " << index << endl;
      return NULL;
   }
   else
   {
      return this->phenotypename[index];
   }
}

string PedConvert::getMarkerName(int index)
{
   if(index < 0 || index > (int) this->markername.size())
   {
      cout << "Error: Index out of bounds! Value was: " << index << endl;
      return NULL;
   }
   else
   {
      return this->markername[index];
   }
}

// Sex covariate
bool PedConvert::isSexCovariate()
{
   return this->sexCovariate;
}

// males of females or all
string PedConvert::getGender()
{
   return this->gender;
}

// Ignore first phenotype in pedigree file
bool PedConvert::isIgnorePhenotype()
{
   return this->ignorePhenotype;
}

// All Traits
bool PedConvert::areAllTraits()
{
   return this->allTraits;
}

double PedConvert::getDisableMapPosition()
{
   return this->disable_map_position;
}

bool PedConvert::isDiscrete()
{
   return this->discrete;
}

string PedConvert::getProgram()
{
   return this->program;
}

char* PedConvert::getOutputPedigreeFile()
{
   if(this->OutputPedigreeFileName==NULL)
   {
      cout << "NULL!";
      return "poo2";
   }
   else
   {
      return this->OutputPedigreeFileName;
   }
}

char* PedConvert::getOutputLocusFile()
{
   return this->OutputLocusFileName;
}

char* PedConvert::getOutputMapFile()
{
   return this->OutputMapFileName;
}

bool PedConvert::writeLinkageFormatLocusFile()
{
   int type;
   double disease_allele_freq;
   int nliab;


   ifstream modelin(getModelFileName());
   if(!modelin)
   {
      cout << endl;
      cout << "Error: Unable to open model file " << getModelFileName() << " for reading." << endl;
      return false;
   }
   modelin >> type >> disease_allele_freq >> nliab;

   if(type!=0 && type!=1)
   {
      cout << endl;
      cout << "Error in model file: Wrong type in Autosomal/X-Linked identifier: " << type << endl;
      return false;
   }

   if(disease_allele_freq < 0.0 || disease_allele_freq > 1.0)
   {
      cout << endl;
      cout << "Error in model file: Disease allele frequency out of range: " << disease_allele_freq << endl;
      return false;
   }

   if(nliab < 1)
   {
      cout << endl;
      cout << "Error in model file: Number of liability classes < 1: " << nliab << endl;
      return false;
   }

   fstream LocusFile(getOutputLocusFile(),ios_base::out);
   if(LocusFile==NULL)
   {
      cout << endl;
      cout << "Error: Unable to open " << "locus.dat" << " for writing!" << endl;
      return -1;
   }
   LocusFile << getNumberOfMarkers()+1 << " 0 " << type << " 5 << NO. OF LOCI, RISK LOCUS, SEXLINKED (IF 1) PROGRAM" << endl;
   LocusFile << "0 0.0 0.0 0 << MUT LOCUS, MUT MALE, MUT FEM, HAP FREQ (IF 1)";
   LocusFile << endl;
   for(int i = 1 ; i <= getNumberOfMarkers()+1 ; i++)
   {
        LocusFile << i << " ";
   }
   LocusFile << endl;
   LocusFile << "1 2 << AFFECTION, NO. OF ALLELES" << endl;
   LocusFile << 1.0-disease_allele_freq << " " << disease_allele_freq << " << GENE FREQUENCIES" << endl;
   LocusFile << nliab << endl;
   double pen[3];
   for(int i = 0 ; i < nliab ; i++)
   {
      modelin >> pen[0] >> pen[1] >> pen[2];
      for(int j = 0 ; j < 3 ; j++)
      {
         if(pen[j] < 0.0 || pen[j] > 1.0)
         {
            cout << "Error in model file: penetrance out of range: " << pen[j] << endl;
            return false;
         }
      }
      LocusFile << pen[0] << " " << pen[1] << " " << pen[2] << " << PENETRANCES" << endl;

      // X-Linked
      if(type==1)
      {
         modelin >> pen[0] >> pen[1];
         for(int j = 0 ; j < 2 ; j++)
         {
            if(pen[j] < 0.0 || pen[j] > 1.0)
            {
               cout << "Error in model file: penetrance out of range: " << pen[j] << endl;
               return false;
            }
         }
         LocusFile  << " " << pen[0] << " " << pen[1] << " << PENETRANCES" << endl;
      }
   }

   LocusFile.close();

   modelin.close();
   return true;
}

bool PedConvert::writePedCheckNamesFile()
{
   cout << endl;
   cout << "Writing PedCheck names file...";
   fstream PedCheckNamesFile("names.dat",ios_base::out);
   if(PedCheckNamesFile==NULL)
   {
      cout << "Error: Unable to open " << "names.dat" << " for writing!" << endl;
      return false;
   }

   if(isIgnorePhenotype())
   {
      PedCheckNamesFile << "X" << endl;
   }
   for(int index = 0 ; index < getNumberOfMarkersInMapFile() ; index++)
   {
      PedCheckNamesFile << getMarkerName(index);
      if(index < getNumberOfMarkersInMapFile() - 1)
      {
         PedCheckNamesFile << endl;
      }
   }
   PedCheckNamesFile.close();
   cout << "done." << endl;
   return true;
}

// This methods parses command line arguments
bool PedConvert::parseArguments(int argc, char * argv[])
{
   /*
    * If not enought arguments we show usage() and exit program
    */
   if(argc < 9)
   {
      usage();
      return false;
   }

   /*
    * Let's parse argument list...
    */
   string argument;
   for(int index = 1 ; index < argc ; index++)
   {
      argument = argv[index];

      // Pedigree file
      if(argument == "-p" && argc >= index+1)
      {
         setPedigreeFile(argv[index+1]);
         index++;
      }

      // Phenotype file
      else if(argument == "-f" && argc >= index+1)
      {
         setPhenotypeFile(argv[index+1]);
         index++;
      }

      // Map file
      else if(argument == "-m" && argc >= index+1)
      {
         setMapFile(argv[index+1]);
         index++;
      }

      // Model file
      else if(argument == "-i" && argc >= index+1)
      {
         setModelFile(argv[index+1]);
         index++;
      }

      // Number of marker in pedigree file
      else if(argument == "-n" && argc >= index+1)
      {
         int markers = atoi(argv[index+1]);
         if(markers > 0 && markers <= MAX_MARKERS)
         {
            setNumberOfMarkers(markers);
            index++;
         }
         else
         {
            cout << "Error: Illegal number of markers (MAX is " << MAX_MARKERS << "), or " << argv[index+1] << " not a number!" << endl;
            return false;
         }
      }

      // Sex as covariate
      else if(argument == "--sex")
      {
         setSexCovariate();
      }

      // Males or Females
      else if(argument == "--only" && argc >= index+1)
      {
         string gender = argv[index+1];
         if(gender == "males" || gender == "females")
         {
            setGender(gender);
         }
         else
         {
            cout << "Error: Illegal gender... : " << gender << endl;
            return false;
         }
         index++;
      }

      // Ignore first phenotype (Dichotomous or QT) on pedigree file
      else if(argument == "--ignore")
      {
         setIgnoreTrait();
      }

      // All phenotypes are quantitative traits
      else if(argument == "--all")
      {
         setAllTraits();
      }

      // All traits are qualitative
      else if(argument == "--discrete")
      {
         this->discrete = true;
      }

      // disable with some map posiotion
      else if(argument == "--dm" && argc >= index+1)
      {
         this->disable_map_position = atof(argv[index+1]);
         index++;
      }

      // Output format option
      else if(argument == "--prog" && argc >= index+1)
      {
         string program = argv[index+1];
         if(program == "solar" || program == "merlin" || program == "genehunter" || program == "analyze")
         {
            setProgram(program);
         }
         else
         {
            cout << "Error: Illegal program name... : " << program << endl;
            return false;
         }
         index++;
      }

      // If argument is illegal...
      else
      {
         cout << "Error: Illegal command line argument \"" << argument << "\"" << endl;
         cout << endl;
         return false;
      }
   }

   // Everything OK!
   return true;
}

// prints "MetaData" info...file names and number of variables...
void PedConvert::print()
{
   cout << " Pedigree  input file      : " << getPedigreeFileName() << endl;
   cout << " Phenotype input file      : " << getPhenotypeFileName() << endl;
   cout << " Map input file            : " << getMapFileName() << endl;
   cout << " Model input file          : " << getModelFileName() << endl;
   cout << " Number of Markers         : " << getNumberOfMarkers() << endl;
   cout << " Sex as Covariate          : ";
   if(isSexCovariate() && getGender() == "all")
   {
     cout << "YES" << endl;
   }
   else
   {
      cout << "NO" << endl;
   }
   cout << " Phenotypes used           : " << getGender() << endl;

   cout << " Dicotomous trait column   : ";
   if(isIgnorePhenotype())
   {
     cout << "YES" << endl;
   }
   else
   {
      cout << "NO" << endl;
   }

   cout << " All are set to traits     : ";
   if(areAllTraits())
   {
     cout << "YES" << endl;
   }
   else
   {
      cout << "NO" << endl;
   }


}

// welcome message
void PedConvert::welcome()
{
   cout << "-------------------------------------------------------------------------------" << endl;
   cout << " Program PedConvert 1.4 - 13th of January 2005" << endl;
   cout << " Copyright (C) 2004-2005 Tero Hiekkalinna, Tero.Hiekkalinna@Helsinki.FI" << endl;
   cout << " Program for converting genotype/phenotype data to ANALYZE, GENEHUNTER, MERLIN" << endl;
   cout << " and SOLAR formats. PedConvert uses Red-Black-Tree data structures." << endl;
   cout << "-------------------------------------------------------------------------------" << endl;
   cout << endl;
}

// prints usage
void PedConvert::usage()
{
   cout << "Usage: pedconvert  [options]" << endl;
   cout << endl;
   cout << " -p     pedigree  file" << endl;
   cout << " -f     phenotype file" << endl;
   cout << " -m     map file" << endl;
   cout << " -i     model file" << endl;
   cout << " -n     number of markers in pedigree file" << endl;

   cout << endl;
   cout << "Input/Output file control:" << endl;
   cout << endl;
   cout << " --prog      [analyze,genehunter,merlin,solar]" << endl;
   cout << " --ingore    Igrore first phenotype from pedigree file" << endl;
   cout << "             (Assumes only one trait phenotype before markers)" << endl;
   cout << " --only      [males,females]" << endl;
   cout << " --discrete  All phenotypes in phenotype file are qualitative traits" << endl;
   cout << endl;
   cout << "Merlin specific options:" << endl;
   cout << endl;
   cout << " --sex       Add sex as covariate" << endl;
   cout << " --dm        [number (cM)]  Disable marker from analysis with specific map" << endl;
   cout << "             distance. E.g. -99.99" << endl;
   cout << endl;
   cout << "Other options:" << endl;
   cout << endl;
   cout << " --all       All phenotypes in phenotype file are quantitative traits" << endl;
   cout << endl;
}

/*
 PedConvert Init Method: This executes all required methods
*/
int init( int argc, char * argv[])
{
   PedConvert MetaData;
   MetaData.welcome();

   // parse arguments
   bool parseOK = MetaData.parseArguments(argc,argv);
   if(!parseOK)
   {
      return -1;
   }

   // print metadata
   MetaData.print();

   // Initialize Pedigree Red-Black Tree
   PedigreeTree *Families = new PedigreeTree();
   Families->setMetaData(&MetaData);

   // This was only used for comparing Red-BLack Tree and Linked Lists
   //PedigreeList *Families = new PedigreeList();

   /*
    * Open pedigree file for reading
    */
   cout << endl;
   cout << "Reading pedigree file...";

   ifstream pedin(MetaData.getPedigreeFileName());
   if(!pedin)
   {
      cout << "Error: Unable to open pedigree file " << MetaData.getPedigreeFileName() << " for reading." << endl;
      return -1;
   }

   /*
    * Let's read pedigree file
    */

   // Temporary variables
   Pedigree *pedigree = new Pedigree("Dummy");
   Person   *person;
   string familyid, personid, fatherid, motherid, sex, affection_status;
   string allele[2];
   bool insertOK;
   string last_familyid = "-1";

   while(!pedin.eof())
   {
      // read person information
      pedin >> familyid >> personid >> fatherid >> motherid >> sex;

      // read dicotomous trait column
      if(MetaData.isIgnorePhenotype())
      {
         pedin >> affection_status;
      }

      /*
       * This is needed or last record is inserted 2 times
       */
      if(pedin.eof())
         break;

      /*
       * If just read familyid is different than previous we allocate for new node
       * and insert it.
       */
      if(familyid != last_familyid)
      {
         // Allocate and insert
         pedigree = new Pedigree(familyid);

         // check allocation
         if(pedigree==NULL)
         {
            cout << "Error: Cannot allocate memory for pedigree, aborting..." << endl;
            return -1;
         }

         insertOK = Families->insert(pedigree);

         /*
          * If pedigree already exists, get pointer to that pedigree
          */
         if(!insertOK)
         {
            pedigree = Families->find(familyid);
         }
      }

      /*
       * Allocate person and insert. Insert is not done if person exists already
       * in family
       */
      person = new Person(familyid,personid,fatherid,motherid,sex);

      // check allocation
      if(person==NULL)
      {
         cout << "Error: Cannot allocate memory for person, aborting..." << endl;
         return -1;
      }

      pedigree->insert(person);

      /*
       * Also we insert MetaData pointer to Person, this is needed in writing out
      *  Merlin formatted input file
       */
       person->setMetaData(&MetaData);

      /*
       * Next we read genotypes (number_of_markers * 2, since each genotype
       * has two alleles)
       */
      for(int marker = 0 ; marker < MetaData.getNumberOfMarkers() ; marker++)
      {
         pedin >> allele[0] >> allele[1];
         person->insertGenotype(allele[0],allele[1]);
      }

      // Let's save last read family id
      last_familyid = familyid;
   }
   pedin.close();
   cout << "done" << endl;

   /*
    * Print pedigree statistics
    */
   cout << endl;
   cout << "Pedigree statistics:" << endl;
   cout << endl;
   cout << "   Number of pedigrees   : " << Families->size() << endl;
   cout << "   Number of individuals : " << Families->sizeTotal() << endl;


   /*
    * Open phenotype file for reading
    */
   cout << endl;
   cout << "Reading phenotype file...";

   ifstream phenin(MetaData.getPhenotypeFileName());
   if(!phenin)
   {
      cout << "Error: Unable to open phenotype file " << MetaData.getPhenotypeFileName() << " for reading." << endl;
      return -1;
   }

   /*
    * Let's read phenotype file
    */
   // Temporary variables
   int number_of_phenotypes;
   int phenotypes_inserted, phenotypes_ignored;
   string value, name;
   char line[10001];
   bool isOK;

   /*
    * Let's read head line of phenotype file, which includes number of phenotypes
    * in file and names of the phenotypes
    */
   phenin >> number_of_phenotypes;
   isOK = MetaData.setNumberOfPhenotypes(number_of_phenotypes);
   if(!isOK)
   {
      return -1;
   }

   // read phenotypes
   for(int index = 0 ; index < MetaData.getNumberOfPhenotypes() ; index++)
   {
      phenin >> name;
      MetaData.addPhenotypeName(name);
   }

   /*
    * Let's read actual phenotype file
    */
   phenotypes_inserted=0;
   phenotypes_ignored=0;
   bool duplicate_phenotype = false;
   while(!phenin.eof())
   {
      // person info
      phenin >> familyid >> personid;

      /*
       * This is needed or last record is inserted 2 times
       */
      if(phenin.eof())
         break;

      /*
       * Next we get person from Red-Black Tree
       */
      pedigree = Families->find(familyid);

      /*
       * If pedigree not found, we show warning and ignore this line
       */
      if(pedigree == NULL)
      {
         phenotypes_ignored += 1;
         cout << "Error: pedigree " << familyid << " is not in pedigree file! ";
         cout << "Ignoring this phenotype record." << endl;
         phenin.getline(line,10000);
      }
      else
      {
         /*
          * Let's get person from pedigree
          */
         person = pedigree->find(personid);

         /*
          * If person not found from pedigree, we show warning and ignore this line
          */
         if(person == NULL)
         {
            phenotypes_ignored += 1;
            cout << "Error: person " << personid << " is not member of pedigree " << familyid << "! ";
            cout << "Ignoring this phenotype record." << endl;
            phenin.getline(line,10000);
         }
         else
         {
            phenotypes_inserted += 1;
            /*
             * Next we read phenotypes and insert them
             */
            for(int index = 0 ; index < MetaData.getNumberOfPhenotypes() ; index++)
            {
               phenin >> value;
               duplicate_phenotype = person->insertPhenotype(MetaData.getPhenotypeName(index),value);
               if(duplicate_phenotype == false){
                  cout << endl;
                  cout << "Error: Duplicate in phenotype file: Family " << familyid << endl;
                  cout << "                                    Person " << personid << endl;
                  cout << "                            Phenotype Name " << MetaData.getPhenotypeName(index) << endl;
                  cout << "Error: Ignoring duplicate phenotype, first one is used!" << endl;
                  phenotypes_ignored += 1;
               }
            }
         }
      }
   }
   phenin.close();
   cout << "done" << endl;

   cout << endl;
   cout << "Phenotype statistics:" << endl;
   cout << endl;
   cout << "   Number of phenotypes inserted : " << phenotypes_inserted << endl;
   cout << "   Number of phenotypes ignored  : " << phenotypes_ignored << endl;
   cout << "   Total number of phenotypes    : " << phenotypes_inserted + phenotypes_ignored << endl;

   /*
    * Map file
    */
   vector<Map> map;
   Map marker_pos;
   string chr, mname;
   double position;
   bool isMarker;
   ifstream mapin(MetaData.getMapFileName());
   if(!mapin)
   {
      cout << "Error: Unable to open map file " << MetaData.getMapFileName() << " for reading." << endl;
      return -1;
   }
   mapin.getline(line,10000); // Header
   while(!mapin.eof())
   {
      mapin >> chr >> position >> mname;

      /*
       * This is needed or last record is inserted 2 times
       */
      if(mapin.eof())
         break;

      isMarker = false;
      for(unsigned int i = 0 ; i < map.size() ; i++)
      {
         if(mname == map[i].getName())
         {
            isMarker = true;
         }
      }

      if(isMarker==true)
      {
         cout << "Warning: Marker \"" << mname << "\" already read (ignoring marker), double entry in map file?" << endl;
      }
      else
      {
         marker_pos = Map(chr,position,mname);
         MetaData.addMarkerName(mname);
         map.push_back(marker_pos);
      }

   }
   mapin.close();

   /*
    * PedCheck names file
    */
   MetaData.writePedCheckNamesFile();

   /*
    *
    */
   if(MetaData.getProgram()=="analyze")
   {
      MetaData.setOutputPedigreeFile("analyze.ped");
      cout << endl;
      cout << "Writing Analyze formatted pedigree file...(LINKAGE)";
      Families->writeLinkageFormat();
      cout << "done." << endl;

   }
   else if(MetaData.getProgram()=="genehunter")
   {
      /*
       * Output files
       */
      MetaData.setOutputPedigreeFile("gh.ped");
      cout << endl;
      cout << "Writing Genehunter formatted pedigree file...(LINKAGE)";
      Families->writeLinkageFormat();
      cout << "done." << endl;
   }
   else if(MetaData.getProgram()=="merlin")
   {
      /*
       * Output files
       */
      MetaData.setOutputPedigreeFile("merlin.ped");
      MetaData.setOutputLocusFile("merlin.dat");
      MetaData.setOutputMapFile("merlin.map");

      /*
       *  Merlin formatted pedigree file
       */
      cout << endl;
      cout << "Writing Merlin formatted pedigree file...";
      Families->writeLinkageFormat();
      cout << "done." << endl;

      /*
       * Merlin map file
      */
      cout << "Writing Merlin formatted map file...";
      fstream MerlinMapFile("merlin.map",ios_base::out);
      if(MerlinMapFile==NULL)
      {
         cout << "Error: Unable to open " << "merlin.map" << " for writing!" << endl;
         return -1;
      }

      // This will print full float field to position column
      MerlinMapFile.setf(ios_base::fixed,ios_base::floatfield);

      for(unsigned int i = 0 ; i < map.size() ; i++)
      {
         MerlinMapFile << map[i].getChr() << " " << map[i].getName() << " " << map[i].getPosition() << endl;
      }
      MerlinMapFile.close();
      cout << "done." << endl;

      /*
       * Merlin locus file
      */
      cout << "Writing Merlin formatted locus file...";
      fstream MerlinLocusFile("merlin.dat",ios_base::out);
      if(MerlinLocusFile==NULL)
      {
         cout << "Error: Unable to open " << "merlin.dat" << " for writing!" << endl;
         return -1;
      }
      if(MetaData.isDiscrete()==true)
      {
         for(int index = 0 ; index < MetaData.getNumberOfPhenotypes() ; index ++)
         {
            MerlinLocusFile << "A " << MetaData.getPhenotypeName(index) << endl;
         }
      }
      else
      {
         for(int index = 0 ; index < MetaData.getNumberOfPhenotypes() ; index ++)
         {
            if(index==0 || MetaData.areAllTraits())
            {
               MerlinLocusFile << "T " << MetaData.getPhenotypeName(index) << endl;
            }
            else
            {
               MerlinLocusFile << "C " << MetaData.getPhenotypeName(index) << endl;
            }
         }
         if(MetaData.isSexCovariate() && MetaData.getGender() == "all")
         {
            MerlinLocusFile << "C Sex" << endl;
         }
      }
      for(unsigned int i = 0 ; i < map.size() ; i++)
      {
         if(map[i].getPosition() == MetaData.getDisableMapPosition())
         {
            cout << "Warning: Marker " << map[i].getName() << " is disabled from analysis!" << endl;
            MerlinLocusFile << "S2 " << map[i].getName() << endl;
         }
         else
         {
            MerlinLocusFile << "M " << map[i].getName() << endl;
         }
      }
      MerlinLocusFile.close();
      cout << "done." << endl;

   /*
    * Merlin model file
    */
    if (MetaData.isModelFileSet()){

      int type;
      double disease_allele_freq;
      int nliab;

      cout << "Reading MODEL file...(" << MetaData.getModelFileName() << ")...";
      ifstream modelin(MetaData.getModelFileName());
      if(!modelin)
      {
         cout << endl;
         cout << "Error: Unable to open model file " << MetaData.getModelFileName() << " for reading." << endl;
         return -1;
      }

      modelin >> type >> disease_allele_freq >> nliab;

      if(type!=0 && type!=1)
      {
         cout << endl;
         cout << "Error in model file: Wrong type in Autosomal/X-Linked identifier: " << type << endl;
         return -1;
      }

      if(disease_allele_freq < 0.0 || disease_allele_freq > 1.0)
      {
         cout << endl;
         cout << "Error in model file: Disease allele frequency out of range: " << disease_allele_freq << endl;
         return -1;
      }

      if(nliab < 1)
      {
         cout << endl;
         cout << "Error in model file: Number of liability classes > 1: " << nliab << endl;
         cout << "This version of AUTOGSCAN only supports one liab class" << endl;
         return -1;
      }
      double pen[3];
      modelin >> pen[0] >> pen[1] >> pen[2];
      for(int j = 0 ; j < 3 ; j++){
        if(pen[j] < 0.0 || pen[j] > 1.0)
        {
            cout << "Error in model file: penetrance out of range: " << pen[j] << endl;
            return -1;
        }
      }
      
      /*
      * Merlin model file
      */
      cout << "Writing Merlin formatted model file...";
      string merlin_model_file_name = "param.tbl";
      fstream MerlinModelFile(merlin_model_file_name.c_str(),ios_base::out);
      if(MerlinModelFile==NULL)
      {
      cout << "Error: Unable to open " << merlin_model_file_name << " for writing!" << endl;
         return -1;
      }
      string name = MetaData.getPhenotypeName(0);
      //MerlinModelFile << "DISEASE             ALLELE_FREQ     PENETRANCES         LABEL" << endl;
      MerlinModelFile << name << "\t" << disease_allele_freq << "\t" << pen[0] << "," << pen[1] << "," << pen[2] << "\t" << name << "_MODEL" << endl;
      MerlinModelFile.close();
      cout << "done." << endl;
   }


   }
   else if(MetaData.getProgram()=="solar")
   {
      // SOLAR
      MetaData.setOutputPedigreeFile("solar.ped");
      MetaData.setOutputPhenotypeFile("solar.phen");
      MetaData.setOutputMarkerFile("solar.marker");
      MetaData.setOutputMapFile("solar.map");

      /*
       *  Solar formatted pedigree file
       */
      cout << endl;
      cout << "Writing Solar formatted pedigree file...";
      Families->writeSolarFormatPedigree();
      cout << "done." << endl;

      cout << "Writing Solar formatted marker file...";
      Families->writeSolarFormatMarkerFile();
      cout << "done." << endl;

      cout << "Writing Solar formatted phenotype file...";
      Families->writeSolarFormatPhenotypeFile();
      cout << "done." << endl;

      /*
      * Solar map file
      */
      cout << "Writing Solar formatted map file...";
      fstream SolarMapFile(MetaData.getOutputMapFile(),ios_base::out);
      if(SolarMapFile==NULL)
      {
      cout << "Error: Unable to open " << MetaData.getOutputMapFile() << " for writing!" << endl;
      return -1;
      }

      // This will print full float field to position column
      SolarMapFile.setf(ios_base::fixed,ios_base::floatfield);

      if(map.size()>0)
      {
         SolarMapFile << map[0].getChr() << " Haldane" << endl;
      }

      for(unsigned int i = 0 ; i < map.size() ; i++)
      {
         SolarMapFile << map[i].getName() << " " << map[i].getPosition() << endl;
      }
      SolarMapFile.close();
      cout << "done." << endl;
   }
   else
   {
      cout << endl;
      cout << "Error: Output format not specified: " << MetaData.getProgram() << endl;
      return -1;
   }
   return 0;
}

/*
  PedConvert Main Method
*/
int main( int argc, char * argv[])
{
   int return_value = 0;

#ifdef _TEST_PEDCONVERT_
   // Test PedConvert classes
   make_all_tests();
#else
   // PedConvert
   return_value = init(argc,argv);
#endif

   //make_test_input_files();

   return return_value;
}
