/*
  Title     : PedigreeTree.cpp
  Version   : 1.0
  Copyright : 2004 Tero Hiekkalinna
  Author    : Tero Hiekkalinna (tero.hiekkalinna@helsinki.fi)
  Date      : 4.2.2004

  58161-5 Tietorakenteiden Harjoitusty, kevt 2004
  Helsingin yliopisto, Tietojenksittelytieteen laitos
  Ohjaaja: Olaf Laczak
*/

#include <iostream>
using namespace std;
#include <fstream>

#include "PedigreeTree.h"
#include "Utils.h"

// Merlin format output file name
#define MERLIN_PEDIGREE_FILE  "merlin.ped"

// Constructor
PedigreeTree::PedigreeTree()
{
   this->root = new Pedigree("NIL","BLACK");
}

// Insert pedigree
bool PedigreeTree::insert(Pedigree *z)
{
   return RBInsert(z);
}

/*
 Red-Black Tree methods

 Variable names (z,x,y) are same as in "Indroduction to
 Algorithms", by Thomas H Cormer et al page 278-281, so it's
 easyer to follow with book if one wishes.

*/

// RB-Insert
bool PedigreeTree::RBInsert(Pedigree *z)
{
   Pedigree *y = new Pedigree("NIL","BLACK");
   Pedigree *x = root;

   while(!x->isNIL())
   {
      /*
       * Let's check that we don't insert duplicate family
       */
      if(z->getKey() == x->getKey())
      {
         cout << "Warning: family " << z->getKey() << " already inserted." << endl;
         return false;
      }

      y = x;

      if(z->getKey() < x->getKey())
      {
         x = x->getLeft();
      }
      else
      {
         x = x->getRight();
      }
   }

   z->setParent(y);

   if(y->isNIL())
   {
      this->root = z;
   }
   else
   {
      if(z->getKey() < y->getKey())
      {
         y->setLeft(z);
      }
      else
      {
         y->setRight(z);
      }
   }

   // set Red-Black tree properties
   z->setLeft(new Pedigree("NIL","BLACK"));
   z->setRight(new Pedigree("NIL","BLACK"));
   z->setRED();

   // Fixup
   RBInsertFixup(z);

   return true;
}

// RB-Insert-Fixup
void PedigreeTree::RBInsertFixup(Pedigree *z)
{
   // z's parent is red
   while((z->getParent())->getColor() == "RED")
   {
      Pedigree *parent      = z->getParent();
      Pedigree *grandparent = parent->getParent();

      if(parent == grandparent->getLeft())
      {
         Pedigree *y = grandparent->getRight();

         // z's uncle y is red
         if(y->getColor() == "RED")
         {
            parent->setBLACK();
            y->setBLACK();
            grandparent->setRED();
            z = grandparent;
         }
         else
         {
            // uncle y is black and z is a right child
            if(z == parent->getRight())
            {
               z = z->getParent();
               LeftRotate(z);
            }

            /* Let's get parent and grandparent pointers again
             * to make sure that they are correct after LeftRotate(z) */
            parent = z->getParent();
            grandparent = parent->getParent();

            // uncle y is black and z is a left child
            parent->setBLACK();
            grandparent->setRED();
            RightRotate(grandparent);
         }
      }
      else
      {
         Pedigree *y = grandparent->getLeft();

         // z's uncle y is red
         if(y->getColor() == "RED")
         {
            parent->setBLACK();
            y->setBLACK();
            grandparent->setRED();
            z = grandparent;
         }
         else
         {
            // uncle y is black and z is a left child
            if(z == parent->getLeft())
            {
               z = z->getParent();
               RightRotate(z);
            }

            /* Let's get parent and grand parenr pointers again
             * to make sure that they are correct after RightRotate(z) */
            parent = z->getParent();
            grandparent = parent->getParent();

            // uncle y is black and z is a right child
            parent->setBLACK();
            grandparent->setRED();
            LeftRotate(grandparent);
         }

      }
   }
   root->setBLACK();
}

/*  Left-Rotate

    X                 Y
   / \               / \
  1   Y      -->    X   3
     / \           / \
    2   3         1   2

*/
void PedigreeTree::LeftRotate(Pedigree *x)
{
   Pedigree *y = x->getRight();

   // set y's left subtree to x's right subtree
   x->setRight(y->getLeft());

   if(!(y->getLeft())->isNIL())
   {
      Pedigree *left = y->getLeft();
      left->setParent(x);
   }

   // y's parent is now x's parent
   y->setParent(x->getParent());

   if((x->getParent())->isNIL())
   {
      this->root = y;
   }
   else
   {
      Pedigree *w = x->getParent();
      if(x == w->getLeft())
      {
         w->setLeft(y);
      }
      else
      {
         w->setRight(y);
      }
   }
   // y's left is now x
   y->setLeft(x);
   // x's parent is now y
   x->setParent(y);
}

/*  Right-Rotate

   X                Y
  / \              / \
 1   Y     <--    X   3
    / \          / \
   2   3        1   2

*/
void PedigreeTree::RightRotate(Pedigree *x)
{
   Pedigree *y = x->getLeft();

   // set y's right subtree to x's letf subtree
   x->setLeft(y->getRight());

   if(!(y->getRight())->isNIL())
   {
      Pedigree *right = y->getRight();
      right->setParent(x);
   }

   // y's parent is now x's parent
   y->setParent(x->getParent());

   if((x->getParent())->isNIL())
   {
      this->root = y;
   }
   else
   {
      Pedigree *w = x->getParent();
      if(x == w->getLeft())
      {
         w->setLeft(y);
      }
      else
      {
         w->setRight(y);
      }
   }
   // y's right is now x
   y->setRight(x);
   // x's parent is y
   x->setParent(y);
}

// finds pedigree with key (familyid) if not found returns NULL
Pedigree* PedigreeTree::find(string key)
{
   Pedigree *x = root;

   while(!x->isNIL())
   {
      if(x->getKey() == key)
         return x;

      if(key < x->getKey())
      {
         x = x->getLeft();
      }
      else
      {
         x = x->getRight();
      }
   }
   return (Pedigree *) NULL;
}

// counts number of pedigrees
int PedigreeTree::size()
{
   this->total_pedigrees = 0;
   size(this->root);
   return this->total_pedigrees;
}

// this method is used by size() method
void PedigreeTree::size(Pedigree *root)
{
   if(root->isNIL())
   {
      return;
   }
   else
   {
      size(root->getRight());
      this->total_pedigrees += 1;
      size(root->getLeft());
   }
}

// counts number of people in all pedigrees combined
int PedigreeTree::sizeTotal()
{
   this->total_individuals = 0;
   sizeTotal(this->root);
   return this->total_individuals;
}

// this method is used by sizeTotal()
void PedigreeTree::sizeTotal(Pedigree *root)
{
   if(root->isNIL())
   {
      return;
   }
   else
   {
      sizeTotal(root->getRight());
      this->total_individuals += root->size();
      sizeTotal(root->getLeft());
   }
}

// prints pedigree Red-Black Tree, used for debuggin tree
void PedigreeTree::print()
{
   RBprint(this->root,"");
}

// prints pedigree Red-Black Tree, used for debuggin tree
void PedigreeTree::RBprint(Pedigree *root, string s)
{
   if(root->isNIL())
   {
      return;
   }
   else
   {
      RBprint(root->getRight(), s + "     |");
      cout << endl;
      cout << s << "---->" << root->getKey() << " (" << root->getColor() << ")";
      cout << endl;
      RBprint(root->getLeft(),  s + "     |");
   }
}

// writes Merlin formatted pedigree file
void PedigreeTree::writeLinkageFormat()
{
   // all pedigrees to string stream
   writeLinkageFormat(root);

   // open output file
   fstream MerlinPedigreeFile(MetaData->getOutputPedigreeFile(),ios_base::out);
   if(MerlinPedigreeFile==NULL)
   {
      cout << "Error: Unable to open " << MetaData->getOutputPedigreeFile() << " for writing!" << endl;
      return;
   }

   // string stream to file
   MerlinPedigreeFile << pedigrees_string.str();
   MerlinPedigreeFile.close();

}

// print each pedigree to string stream
void PedigreeTree::writeLinkageFormat(Pedigree *root)
{
   if(root->isNIL())
   {
      return;
   }
   else
   {
      writeLinkageFormat(root->getLeft());
      this->pedigrees_string << root->getInString();
      writeLinkageFormat(root->getRight());
   }
}

void PedigreeTree::writeSolarFormatPedigree()
{
   // Header
   this->pedigrees_string << "famid,id,fa,mo,sex" << endl;
   // all pedigrees to string stream
   writeSolarFormatPedigree(root);
   // open output file
   fstream SolarPedigreeFile(MetaData->getOutputPedigreeFile(),ios_base::out);
   if(SolarPedigreeFile==NULL)
   {
      cout << "Error: Unable to open " << MetaData->getOutputPedigreeFile() << " for writing!" << endl;
      return;
   }
   // string stream to file
   SolarPedigreeFile << pedigrees_string.str();
   SolarPedigreeFile.close();
}

// print each pedigree to string stream
void PedigreeTree::writeSolarFormatPedigree(Pedigree *root)
{
   if(root->isNIL())
   {
      return;
   }
   else
   {
      writeSolarFormatPedigree(root->getLeft());
      this->pedigrees_string << root->getInSolarString();
      writeSolarFormatPedigree(root->getRight());
   }
}

void PedigreeTree::writeSolarFormatMarkerFile()
{
   solarmarker_string << "famid,id";
   for(int index = 0 ; index < MetaData->getNumberOfMarkersInMapFile() ; index ++)
   {
      solarmarker_string << "," << MetaData->getMarkerName(index);
   }
   solarmarker_string << endl;
   writeSolarFormatMarkerFile(root);
   // open output file
   fstream SolarMarkerFile(MetaData->getOutputMarkerFile(),ios_base::out);
   if(SolarMarkerFile==NULL)
   {
      cout << "Error: Unable to open " << MetaData->getOutputMarkerFile() << " for writing!" << endl;
      return;
   }
   // string stream to file
   SolarMarkerFile << solarmarker_string.str();
   SolarMarkerFile.close();
}

// print each pedigree to string stream
void PedigreeTree::writeSolarFormatMarkerFile(Pedigree *root)
{
   if(root->isNIL())
   {
      return;
   }
   else
   {
      writeSolarFormatMarkerFile(root->getLeft());
      this->solarmarker_string << root->getMarkersInSolarString();
      writeSolarFormatMarkerFile(root->getRight());
   }
}

void PedigreeTree::writeSolarFormatPhenotypeFile()
{
   solarphenotype_string << "famid,id";
   for(int index = 0 ; index < MetaData->getNumberOfPhenotypes() ; index ++)
   {
      solarphenotype_string << "," << MetaData->getPhenotypeName(index);
   }
   solarphenotype_string << endl;
   writeSolarFormatPhenotypeFile(root);
   // open output file
   fstream SolarPhenotypeFile(MetaData->getOutputPhenotypeFile(),ios_base::out);
   if(SolarPhenotypeFile==NULL)
   {
      cout << "Error: Unable to open " << MetaData->getOutputPhenotypeFile() << " for writing!" << endl;
      return;
   }
   // string stream to file
   SolarPhenotypeFile << solarphenotype_string.str();
   SolarPhenotypeFile.close();
}

// print each pedigree to string stream
void PedigreeTree::writeSolarFormatPhenotypeFile(Pedigree *root)
{
   if(root->isNIL())
   {
      return;
   }
   else
   {
      writeSolarFormatPhenotypeFile(root->getLeft());
      this->solarphenotype_string << root->getPhenotypesInSolarString();
      writeSolarFormatPhenotypeFile(root->getRight());
   }
}

// set metadata
void PedigreeTree::setMetaData(PedConvert *MetaData)
{
   this->MetaData = MetaData;
}

